/*
 * Copyright (c) 2015 by inovat, innovative systeme - verkehr - tunnel - technik, Dipl.-Ing. H. C. Kniss
 *
 * This file is part of de.bsvrz.kex.tls.osi7.conversion.FgAllDefault
 *
 * de.bsvrz.kex.tls.osi7.conversion.FgAllDefault is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  de.bsvrz.kex.tls.osi7.conversion.FgAllDefault is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with de.bsvrz.kex.tls.osi7.conversion.FgAllDefault.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Diese Datei ist Teil von de.bsvrz.kex.tls.osi7.conversion.FgAllDefault.
 *
 *  de.bsvrz.kex.tls.osi7.conversion.FgAllDefault ist Freie Software: Sie können es unter den Bedingungen
 *  der GNU General Public License, wie von der Free Software Foundation,
 *  Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
 *  veröffentlichten Version, weiterverbreiten und/oder modifizieren.
 *
 *  de.bsvrz.kex.tls.osi7.conversion.FgAllDefault wird in der Hoffnung, dass es nützlich sein wird, aber
 *  OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 *  Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
 *  Siehe die GNU General Public License für weitere Details.
 *
 *  Sie sollten eine Kopie der GNU General Public License zusammen mit diesem
 *  Programm erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 *
 * Contact Information:
 * inovat, Dipl.-Ing. H. C. Kniss
 * An der Krautwiese 37
 * D-53783 Eitorf
 * +49 (0)2243 8464 193
 * info@inovat.de
 * www.inovat.de
 */

package de.bsvrz.kex.tls.osi7.conversion;

import de.bsvrz.dav.daf.main.*;
import de.bsvrz.dav.daf.main.config.IntegerValueState;
import de.bsvrz.kex.tls.osi7.*;
import de.bsvrz.sys.funclib.debug.Debug;
import de.bsvrz.sys.funclib.hexdump.HexDumper;

/**
 * Umsetungsmodul für die alle FG. Es werden KEINE Umsetzungen angemeldet. Umsetzungsmodul enthält alle
 * Umsetzungsmethoden, die in allen FG identisch sind (z.B. Geografische Kenndaten). Von diesr Klasse werden
 * normalerweise alle anderen konkreten Umsetzungsklassen abgeleitet.
 *
 * @author inovat, innovative systeme - verkehr - tunnel - technik
 * @author inovat, innovative systeme - verkehr - tunnel - technik
 * @version $Revision$ / $Date$ / ($Author$)
 */

public class FgAllDefault extends DeConversionModul {
// ------------------------------------------------------------------------
// VARIABLENDEKLARATIONEN
// ------------------------------------------------------------------------

// Sonstige ---------------------------------------------------------------
	/** DebugLogger für Debug-Ausgaben */
	private static final Debug debug = Debug.getLogger();

// ------------------------------------------------------------------------
// KONSTRUKTOREN  (und vom Konstruktor verwende Methoden)
// ------------------------------------------------------------------------

	/**
	 * Erzeugt ein Objekt vom Typ Fg001Default
	 *
	 * @param daf DaV
	 * @param eak Eak
	 * @param de De
	 */
	public FgAllDefault(ClientDavInterface daf, Eak eak, De de) {
		super(daf, eak, de);
	}

	/** ACHTUNG: Nur zum Testen!. */
	public FgAllDefault() {
		
	}

	// ------------------------------------------------------------------------
// SONSTIGE METHODEN
// ------------------------------------------------------------------------

	/**
	 * FG alle : Typ: 1 Id: 1 Name: DE-Fehler (Antwort). TLS-Datenbeschreibung:
	 * <pre>
	 * Position          Bezeichnung                             Erläuterung
	 * Byte 1            Länge DE-Block                          Länge des folgenden DE-Blocks [ 4 ]
	 * Byte 2            Daten-Endgeräte-Kanal                   [ 1.. 254 ]
	 * Byte 3            Typ der DE-Daten                        [ 1 ]
	 * Byte 4            Fehlercode siehe unten
	 * Byte 5            Herstellercode                          siehe Liste in Anhang 7, Kap. 2
	 * </pre>
	 * Inhalt Byte: Fehlercode:
	 * <pre>
	 *    7        6        5        4        3        2        1        0
	 *    0        0        0        0        Passiv   Proj-F   Stö/SM   Stö/EAK
	 *
	 * Bit 0:    Stö/EAK =  0: Keine Störung / vom E/A-Konzentrator erkannt
	 *                      1: Störung / vom E/A-Konzentrator erkannt
	 * Bit 1:    Stö/SM =   0: Keine Störung / vom SM erkannt
	 *                      1: Störung / vom SM erkannt
	 * Bit 2:    Proj-F =   0: Projektierungsdaten für diesen Kanal sind gültig
	 *                      1: Projektierungsdaten für diesen Kanal enthalten ungültige Werte
	 * Bit 3:    Passiv =   0: Der Kanal ist nicht passiviert
	 *                      1: Der Kanal ist passiviert
	 * Bit 4..7:        =   0: reserviert
	 * </pre>
	 * Die Bits 0 und 1 werden nur einzeln gesetzt, d.h. es darf nur maximal ein Bit den Zustand 1 annehmen. <br> Inhalt
	 * Byte Herstellercode:<br> Der Herstellercode ist ein für jeden Hersteller eindeutig zugeordneter Code, der der
	 * Zentrale die Zuordnung der Nachrichten erleichtern soll. Der Herstellercode bezieht sich auf den Hersteller des die
	 * Nachricht erzeugenden Prozesses, nicht auf das gestörte Gerät.<br>
	 *
	 * @param deBlock         Der zu bearbeitende De-Block.
	 * @param data            Der passende Datensatz für die Ziel Attributgruppe/Aspekt-Kombination.
	 * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
	 *
	 * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
	 */
	public ResultData Fg255Typ001Id001AtgTlsGloDeFehlerAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
		ResultData resultData = null;

		// Fehlercode-Byte aus Telegramm -------------------------------------------------------------------------------
		int _Fehlercode_ = deBlock.getDeData().readUnsignedByteAsInt();

		_de.setOk(false);// Fehlerzustand der DE auf "defekt setzen".
		switch (_Fehlercode_ & 0x03) {
			case 0:
				data.getUnscaledValue("DEFehlerStatus").setText("ok");
				_de.setOk(true);// nur bei De-Gutmeldung Fehlerzustand der De auf "ok" setzen.
				break;
			case 1:
				data.getUnscaledValue("DEFehlerStatus").setText("StörEAK");
				break;
			case 2:
				data.getUnscaledValue("DEFehlerStatus").setText("StörSM");
				break;
			default:
				debug.warning("DeFehlerStatus ungültig : " + _Fehlercode_ + Debug.NEWLINE +
				              "De-Block (Osi7Adresse-DeNummer-FG-Typ-Id): " + deBlock.getLongKey() + " verworfen:" + Debug.NEWLINE +
				              deBlock.toString());
				return resultData;
		}

		if ((_Fehlercode_ & 0x04) == 0) {
			data.getScaledValue("DEProjektierungsStatus").setText("Projektierung ok");
		}
		else {
			data.getUnscaledValue("DEProjektierungsStatus").setText("Projektierung Fehler");
		}

		if ((_Fehlercode_ & 0x08) == 0) {
			data.getUnscaledValue("DEKanalStatus").setText("aktiv");
		}
		else {
			data.getUnscaledValue("DEKanalStatus").setText("passiv");
		}

		data.getUnscaledValue("HerstellerDefinierterCode").set(_Fehlercode_ >> 4);

		// Herstellercode-Byte aus Telegramm ---------------------------------------------------------------------------
		int _Herstellercode_ = deBlock.getDeData().readUnsignedByteAsInt();

		data.getUnscaledValue("Hersteller").set(_Herstellercode_);

		//Ergebnisdatensatz erzeugen...
		resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);
		return resultData;
	}

	/**
	 * FG alle : Typ: 16 Id: 2 Name: Negative Quittung (Antwort) TLS-Datenbeschreibung:<br>
	 * <pre>
	 * Position         Bezeichnung                             Erläuterung
	 * Byte 1           Länge DE-Block                          Länge des folgenden DE-Blocks [ 4 ]
	 * Byte 2           Daten-Endgeräte-Kanal                   [ 1..254 ]
	 * Byte 3           Typ der DE-Daten                        [ 16 ]
	 * Byte 4           Fehlerursache siehe unten
	 * Byte 5           Herstellercode                          siehe Liste in Anhang 7, Kap. 2
	 * </pre>
	 * Inhalt Byte: siehe Beschreibung bei der jeweiligen FG<br> Inhalt Byte Herstellercode:<br> Der Herstellercode ist ein
	 * für jeden Hersteller eindeutig zugeordneter Code, der der Zentrale die Zuordnung der Nachrichten erleichtern soll.
	 *
	 * @param deBlock         Der zu bearbeitende De-Block.
	 * @param data            Der passende Datensatz für die Ziel Attributgruppe/Aspekt-Kombination.
	 * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
	 *
	 * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
	 */
	public ResultData Fg255Typ016Id002AtgTlsGloNegativeQuittungAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
		ResultData resultData = null;

		// FehlerUrsache-Byte aus Telegramm ----------------------------------------------------------------------------
		int _FehlerUrsache_ = deBlock.getDeData().readUnsignedByteAsInt();
		data.getUnscaledValue("Code").set(_FehlerUrsache_);

		// Herstellercode-Byte aus Telegramm ---------------------------------------------------------------------------
		int _Herstellercode_ = deBlock.getDeData().readUnsignedByteAsInt();
		data.getUnscaledValue("Hersteller").set(_Herstellercode_);

		// Einzeltelegramm, welches negative Quittung auslöste, ermitteln und speichern

		SingleTelegram singleTelegram = TlsApplicationLayer.getSendSingleTelegram(_eakConversionModul.getJobNumber());
		String singleTelegrammText = "Einzeltelegramm, welches negative Quittung auslöste, konnte nicht ermittelt werden!";
		if (singleTelegram != null) {
			singleTelegrammText = singleTelegram.toString();
		}
		data.getTextValue("EinzelTelegramm").setText(singleTelegrammText);

		//Zuordnung zu versandtem Einzeltelegramm, welches negative Quittung auslöste, ausgeben.
		debug.warning("Negative Quittung: \n" +
		              "Hersteller    : " + data.getTextValue("Hersteller").getText() + "\n" +
		              "Fehlerursache : " + data.getTextValue("Code").getText() + "\n" +
		              "Versandtes Einzeltelegramm:\n" +
		              singleTelegrammText);

		//Ergebnisdatensatz erzeugen...
		resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);

		return resultData;
	}

	/**
	 * FG alle : Typ: 29 Id: 2 Name: Kanalsteuerung (Antwort). Umsetzung erfolg in Attributgruppe/Aspekt
	 * <code>atg.tlsGloKanalSteuerung/asp.tlsAntwort</code><br> TLS-Datenbeschreibung:
	 * <pre>
	 * Position         Bezeichnung                             Erläuterung
	 * Byte 1           Länge DE-Block                          Länge des folgenden DE-Blocks [ 3 ]
	 * Byte 2           Daten-Endgeräte-Kanal (DE)              [ 1..254, 255 ]
	 * Byte 3           Typ der DE-Daten                        [ 29 ]
	 * Byte 4           Kanalsteuerbyte siehe unten
	 *
	 * 7       6       5      4       3       2       1       0
	 * frei für Hersteller            0       0       0       Passiv
	 * </pre>
	 * Inhalt Byte Kanalsteuerbyte<br>
	 * <pre>
	 * Bit 0 = 0:       Normalbetrieb
	 * Bit 0 = 1:       Kanal ist passiv / Kanal passivieren
	 * Bit 1...3:       noch nicht definiert
	 * Bit 4...7:       frei für Herstellerdefinitionen
	 * </pre>
	 *
	 * @param deBlock         Der zu bearbeitende De-Block.
	 * @param data            Der passende Datensatz für die Ziel Attributgruppe/Aspekt-Kombination.
	 * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
	 *
	 * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
	 */
	public ResultData Fg255Typ029Id002AtgTlsGloKanalSteuerungAspParameterIst(DeBlock deBlock, Data data, DataDescription dataDescription) {
		ResultData resultData = null;

		// Kanalsteuer-Byte aus Telegramm ------------------------------------------------------------------------------
		int _Kanalsteuerbyte_ = deBlock.getDeData().readUnsignedByteAsInt();

		switch (_Kanalsteuerbyte_ & 0x01) {
			case 0:
				data.getUnscaledValue("DEKanalStatus").setText("aktiv");
				break;
			case 1:
				data.getUnscaledValue("DEKanalStatus").setText("passiv");
				break;
			default:
				debug.warning("DeFKanalStatus ungültig : " + _Kanalsteuerbyte_ + Debug.NEWLINE +
				              "De-Block (Osi7Adresse-DeNummer-FG-Typ-Id): " + deBlock.getLongKey() + " verworfen:" + Debug.NEWLINE +
				              deBlock.toString());
				return resultData;
		}

		data.getUnscaledValue("HerstellerDefinierterCode").set(_Kanalsteuerbyte_ >> 4);

		//Ergebnisdatensatz erzeugen...

		resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);
		return resultData;
	}

	/**
	 * FG alle : Typ: 29 Id: 2 Name: Kanalsteuerung (Senden). Umsetzung erfolg in Attributgruppe/Aspekt
	 * <code>atg.tlsGloKanalSteuerung/asp.tlsAntwort</code><br> TLS-Datenbeschreibung:
	 * <pre>
	 * Position         Bezeichnung                             Erläuterung
	 * Byte 1           Länge DE-Block                          Länge des folgenden DE-Blocks [ 3 ]
	 * Byte 2           Daten-Endgeräte-Kanal (DE)              [ 1..254, 255 ]
	 * Byte 3           Typ der DE-Daten                        [ 29 ]
	 * Byte 4           Kanalsteuerbyte siehe unten
	 *
	 * 7       6       5      4       3       2       1       0
	 * frei für Hersteller            0       0       0       Passiv
	 * </pre>
	 * Inhalt Byte Kanalsteuerbyte<br>
	 * <pre>
	 * Bit 0 = 0:       Normalbetrieb
	 * Bit 0 = 1:       Kanal ist passiv / Kanal passivieren
	 * Bit 1...3:       noch nicht definiert
	 * Bit 4...7:       frei für Herstellerdefinitionen
	 * </pre>
	 *
	 * @param result Der vom Datenverteiler empgangene Datensatz, der in einen De-Block konvertiert wird.
	 *
	 * @return De-Block mit den konvertierten Daten.
	 */
	public DeBlock Fg255Typ029Id002AtgTlsGloKanalSteuerungAspParameterSoll(ResultData result) {
		DeBlock deBlock = null;

		// Datenblock beseht aus folgenden Inhalten:
		int _Kanalsteuerbyte_;

		Data data = result.getData();// Datensatz holen

		IntegerValueState DeKanalStatus = data.getUnscaledValue("DEKanalStatus").getState();
		if (DeKanalStatus != null) {
			if (DeKanalStatus.getName().equals("aktiv")) {
				_Kanalsteuerbyte_ = 0;
			}
			else if (DeKanalStatus.getName().equals("passiv")) {
				_Kanalsteuerbyte_ = 1;
			}
			else {
				debug.warning("Unbekannter DeKanalStatus-Wert: " + DeKanalStatus);
				return deBlock;
			}
		}
		else {
			debug.warning("Unbekannter DeKanalStatus-Wert: " + data.getUnscaledValue("DEKanalStatus").intValue());
			return deBlock;
		}
		_Kanalsteuerbyte_ += data.getUnscaledValue("HerstellerDefinierterCode").intValue() << 4;

		//DeBlock als TlsData aufbauen...

		TlsData deApplicationData = new TlsData(1);
		deApplicationData.writeIntAsByte(_Kanalsteuerbyte_);

		// DeBlock erzeugen
		try {
			deBlock = new DeBlock(deApplicationData, _de.getEak().getSm().getOsi7Adress(), _de.getFg(), 29, 2, _de.getDeChannel());
			debug.finest("TLS-DE-Block: " + HexDumper.toString(deBlock.getTlsDeDataBlock()));
		}
		catch (IllegalOsi7TelegramStructureException e) {
			debug.warning("Fehler beim Erzeugen eines De-Blocks zum Versand: " + e);
		}
		return deBlock;
	}

	/**
	 * FG alle : Typ: 36 Id: 3 Name: Geographische Kenndaten (Antwort). Umsetzung erfolg in Attributgruppe/Aspekt
	 * <code>atg.tlsGloGeographischeKenndaten/asp.tlsZustand</code><br> TLS-Datenbeschreibung:
	 * <pre>
	 * Position         Bezeichnung                             Erläuterung
	 * Byte 1           Länge DE-Block                          Länge des folgenden DE-Blocks
	 * Byte 2           Daten-Endgeräte-Kanal                   [ 0 bzw. entspricht OSI 2-Adresse ]
	 * Byte 3           Typ der DE-Daten                        [ 36 ]
	 * Byte 4           Landeskennung
	 * Byte 5           Straßenart
	 * Byte 6           Straßennummer                           high Byte, Ziffern 4 und 3
	 * Byte 7           Straßennummer                           low Byte,, Ziffern 2 und 1
	 * Byte 8           Kilometrierung                          extra high Byte, Wertigkeit: 103km,102km
	 * Byte 9           Kilometrierung                          high Byte, Wertigkeit: 101km,100km
	 * Byte 10          Kilometrierung                          low Byte, Wertigkeit: 10-1km,10-2km
	 * Byte 11          Fahrtrichtung
	 * Byte 12          Reservebyte
	 * </pre>
	 * Inhalt Byte: Landeskennung:<br>
	 * <pre>
	 * 1:  Schleswig-Holstein
	 * 2:  Hamburg
	 * 3:  Niedersachsen
	 * 4:  Bremen
	 * 5:  Nordrhein-Westfalen
	 * 6:  Hessen
	 * 7:  Rheinland-Pfalz
	 * 8:  Baden-Württemberg
	 * 9:  Bayern
	 * 10: Saarland
	 * 11: Berlin
	 * 12: Brandenburg
	 * 13: Mecklenburg-Vorpommern
	 * 14: Sachsen
	 * 15: Sachsen-Anhalt
	 * 16: Thüringen
	 * </pre>
	 * Inhalt Byte: Straßenart:<br>
	 * <pre>
	 * 1: Bundesautobahn
	 * 2: Bundesstraße
	 * 3: Landesstraße
	 * 4: ..
	 * </pre>
	 * Inhalt Bytes: Straßennummer:<br> Straßennummer 0..9999, in Halbbytes binär codiert<br> Inhalt Bytes:
	 * Kilometrierung:<br> Kilometrierung des Querschnittes 0,00 bis 9999,99km, in Halbbytes binär codiert<br> Inhalt Byte:
	 * Fahrtrichtung:<br>
	 * <pre>
	 * Bit0 = 1: Fahrtrichtung Nord bzw. Ost
	 * Bit1 = 1: Fahrtrichtung Süd bzw. West
	 * Bit2..7 : noch nicht definiert
	 * </pre>
	 * In Streckenstationen, bzw. FGs, die beide Fahrtrichtungen betreffen, sind die Bits 0 und 1 gleichzeitig gesetzt.
	 * Streckenstationen, bzw. FGs, die keine fahrbahnbezogenen Geräte enthalten, setzen beide Bits auf 0. Für die
	 * Fahrtrichtung ist die Ausrichtung der gesamten Straße zwischen ihren Endpunkten entscheidend, nicht die Lage der
	 * Fahrbahn am betrachteten Querschnitt.
	 *
	 * @param result Der vom Datenverteiler empgangene Datensatz, der in einen De-Block konvertiert wird.
	 *
	 * @return De-Block mit den konvertierten Daten.
	 */
	public DeBlock Fg255Typ036Id003AtgTlsGloGeographischeKenndatenAspTlsVorgabe(ResultData result) {
		DeBlock deBlock = null;

		// Datenblock beseht aus folgenden Inhalten:
		int _Landeskennung_;
		int _Strassenart_;
		int _StrassenNummer_;
		int _Kilometrierung_;
		int _Fahrtrichtung_;
		int _Reservebyte_;

		// Umsetzung
		Data data = result.getData();// Datensatz holen

		_Landeskennung_ = data.getUnscaledValue("LandesKennung").intValue();
		_Strassenart_ = data.getUnscaledValue("StraßenArt").intValue();
		_StrassenNummer_ = data.getUnscaledValue("StraßenNummer").intValue();
		_Kilometrierung_ = data.getUnscaledValue("Kilometrierung").intValue();

		if (data.getUnscaledValue("Fahrtrichtung").getState().getName().equals("Nord bzw. Ost")) {
			_Fahrtrichtung_ = 0x01;
		}
		else if (data.getUnscaledValue("Fahrtrichtung").getState().getName().equals("Süd bzw. West")) {
			_Fahrtrichtung_ = 0x02;
		}
		else {
			_Fahrtrichtung_ = 0;
		}

		_Reservebyte_ = data.getUnscaledValue("ReserveByte").intValue();

		// DeBlock als TlsData aufbauen...

		TlsData deApplicationData = new TlsData(9);
		deApplicationData.writeIntAsByte(_Landeskennung_);
		deApplicationData.writeIntAsByte(_Strassenart_);
		deApplicationData.writeIntAsHighLowByteAsHalfByte(_StrassenNummer_);
		deApplicationData.writeIntAsExtraHighHighLowByteAsHalfByte(_Kilometrierung_);
		deApplicationData.writeIntAsByte(_Fahrtrichtung_);
		deApplicationData.writeIntAsByte(_Reservebyte_);

		// DeBlock erzeugen
		try {
			deBlock = new DeBlock(deApplicationData, _de.getEak().getSm().getOsi7Adress(), _de.getFg(), 36, 3, _de.getDeChannel());
		}
		catch (IllegalOsi7TelegramStructureException e) {
			debug.warning("Fehler beim Erzeugen eines De-Blocks zum Versand: " + e);
		}
		return deBlock;
	}

	/**
	 * FG alle : Typ: 36 Id: 3 Name: Geographische Kenndaten (Antwort). Umsetzung erfolg in Attributgruppe/Aspekt
	 * <code>atg.tlsGloGeographischeKenndaten/asp.tlsZustand</code><br> TLS-Datenbeschreibung:
	 * <pre>
	 * Position         Bezeichnung                             Erläuterung
	 * Byte 1           Länge DE-Block                          Länge des folgenden DE-Blocks
	 * Byte 2           Daten-Endgeräte-Kanal                   [ 0 bzw. entspricht OSI 2-Adresse ]
	 * Byte 3           Typ der DE-Daten                        [ 36 ]
	 * Byte 4           Landeskennung
	 * Byte 5           Straßenart
	 * Byte 6           Straßennummer                           high Byte, Ziffern 4 und 3
	 * Byte 7           Straßennummer                           low Byte,, Ziffern 2 und 1
	 * Byte 8           Kilometrierung                          extra high Byte, Wertigkeit: 103km,102km
	 * Byte 9           Kilometrierung                          high Byte, Wertigkeit: 101km,100km
	 * Byte 10          Kilometrierung                          low Byte, Wertigkeit: 10-1km,10-2km
	 * Byte 11          Fahrtrichtung
	 * Byte 12          Reservebyte
	 * </pre>
	 * Inhalt Byte: Landeskennung:<br>
	 * <pre>
	 * 1:  Schleswig-Holstein
	 * 2:  Hamburg
	 * 3:  Niedersachsen
	 * 4:  Bremen
	 * 5:  Nordrhein-Westfalen
	 * 6:  Hessen
	 * 7:  Rheinland-Pfalz
	 * 8:  Baden-Württemberg
	 * 9:  Bayern
	 * 10: Saarland
	 * 11: Berlin
	 * 12: Brandenburg
	 * 13: Mecklenburg-Vorpommern
	 * 14: Sachsen
	 * 15: Sachsen-Anhalt
	 * 16: Thüringen
	 * </pre>
	 * Inhalt Byte: Straßenart:<br>
	 * <pre>
	 * 1: Bundesautobahn
	 * 2: Bundesstraße
	 * 3: Landesstraße
	 * 4: ..
	 * </pre>
	 * Inhalt Bytes: Straßennummer:<br> Straßennummer 0..9999, in Halbbytes binär codiert<br> Inhalt Bytes:
	 * Kilometrierung:<br> Kilometrierung des Querschnittes 0,00 bis 9999,99km, in Halbbytes binär codiert<br> Inhalt Byte:
	 * Fahrtrichtung:<br>
	 * <pre>
	 * Bit0 = 1: Fahrtrichtung Nord bzw. Ost
	 * Bit1 = 1: Fahrtrichtung Süd bzw. West
	 * Bit2..7 : noch nicht definiert
	 * </pre>
	 * In Streckenstationen, bzw. FGs, die beide Fahrtrichtungen betreffen, sind die Bits 0 und 1 gleichzeitig gesetzt.
	 * Streckenstationen, bzw. FGs, die keine fahrbahnbezogenen Geräte enthalten, setzen beide Bits auf 0. Für die
	 * Fahrtrichtung ist die Ausrichtung der gesamten Straße zwischen ihren Endpunkten entscheidend, nicht die Lage der
	 * Fahrbahn am betrachteten Querschnitt.
	 *
	 * @param deBlock         Der zu bearbeitende De-Block.
	 * @param data            Der passende Datensatz für die Ziel Attributgruppe/Aspekt-Kombination.
	 * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
	 *
	 * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
	 */
	public ResultData Fg255Typ036Id003AtgTlsGloGeographischeKenndatenAspTlsZustand(DeBlock deBlock, Data data, DataDescription dataDescription) {
		ResultData resultData = null;

		// Landeskennung aus Telegramm ---------------------------------------------------------------------------------
		int _Landeskennung_ = deBlock.getDeData().readUnsignedByteAsInt();

		if ((_Landeskennung_ >= 1) && (_Landeskennung_ <= 16)) {
			data.getUnscaledValue("LandesKennung").set(_Landeskennung_);
		}
		else {
			debug.warning("Landeskennung ungültig (1..16) : " + _Landeskennung_ + Debug.NEWLINE +
			              "De-Block mit Landeskennung='unbekannt' verarbeitet:" + Debug.NEWLINE +
			              deBlock.toString());
			data.getUnscaledValue("LandesKennung").setText("unbekannt");
		}

		// Straßenart aus Telegramm ------------------------------------------------------------------------------------
		int _Strassenart_ = deBlock.getDeData().readUnsignedByteAsInt();

		if ((_Strassenart_ >= 1) && (_Strassenart_ <= 3)) {
			data.getUnscaledValue("StraßenArt").set(_Strassenart_);
		}
		else {
			data.getUnscaledValue("StraßenArt").setText("sonstige Straße");
		}

		// Straßennummr aus Telegramm ----------------------------------------------------------------------------------
		int _StrassenNummer_ = deBlock.getDeData().readHighLowByteAsHalfByteAsInt();

		data.getUnscaledValue("StraßenNummer").set(_StrassenNummer_);

		// Kilometrierung aus Telegramm --------------------------------------------------------------------------------n
		int _Kilometrierung_ = deBlock.getDeData().readExtraHighHighLowByteAsHalfByteAsInt();

		data.getUnscaledValue("Kilometrierung").set(_Kilometrierung_);

		// Landeskennung aus Telegramm ---------------------------------------------------------------------------------
		int _Fahrtrichtung_ = deBlock.getDeData().readUnsignedByteAsInt();

		switch (_Fahrtrichtung_ & 0x03) {
			case 1:
				data.getUnscaledValue("Fahrtrichtung").setText("Nord bzw. Ost");
				break;
			case 2:
				data.getUnscaledValue("Fahrtrichtung").setText("Süd bzw. West");
				break;
			default:
				debug.warning("Fahrtrichtung ungültig : " + _Fahrtrichtung_ + Debug.NEWLINE +
				              "De-Block mit Fahrtrichtung='nicht richtungsbezogen' verarbeitet:" + Debug.NEWLINE +
				              deBlock.toString());
				data.getUnscaledValue("Fahrtrichtung").setText("nicht richtungsbezogen");
		}

		// Reservebyte aus Telegramm -----------------------------------------------------------------------------------

		int _Reservebyte_ = deBlock.getDeData().readUnsignedByteAsInt();

		data.getUnscaledValue("ReserveByte").set(_Reservebyte_);

		//Ergebnisdatensatz erzeugen...

		resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);
		return resultData;
	}

	/**
	 * FG alle : Typ: alle Id: alle Name: Abruf DE-Block (Senden). Umsetzung erfolg in Attributgruppe/Aspekt
	 * <code>atg.tlsGloDeBlockAbruf/asp.tlsAbruf</code><br> TLS-Datenbeschreibung:
	 * <pre>
	 * Position         Bezeichnung                             Erläuterung
	 * Byte 1           Länge DE-Block                          Länge des folgenden DE-Blocks [ 3 ]
	 * Byte 2           Daten-Endgeräte-Kanal (DE)              [ 1..254, 255 ]
	 * Byte 3           Typ der DE-Daten                        [ alle ]
	 *
	 * </pre>
	 *
	 * @param result Der vom Datenverteiler empgangene Datensatz, der in einen De-Block konvertiert wird.
	 *
	 * @return De-Block mit den konvertierten Daten.
	 */
	public DeBlock Fg255Typ255Id255AtgTlsGloDeBlockAbrufAspTlsAbruf(ResultData result) {
		DeBlock deBlock = null;

		// Datenblock beseht aus folgenden Inhalten:
		int _JobNummer_;//TBD: Jobnummer gehört nicht zum DE-Block sondern zum Einzeltelegramm. Umsetzung ????
		int _ID_;
		int _Typ_;

		Data data = result.getData();// Datensatz holen

		_JobNummer_ = data.getUnscaledValue("JobNummer").intValue();
		_ID_ = data.getUnscaledValue("ID").intValue();
		_Typ_ = data.getUnscaledValue("Typ").intValue();

		//DeBlock als TlsData aufbauen...

		TlsData deApplicationData = new TlsData(0);

		// DeBlock erzeugen
		try {
			deBlock = new DeBlock(deApplicationData, _de.getEak().getSm().getOsi7Adress(), _de.getFg(), _Typ_, _ID_, _de.getDeChannel());
		}
		catch (IllegalOsi7TelegramStructureException e) {
			debug.warning("Fehler beim Erzeugen eines De-Blocks zum Versand: " + e);
		}
		return deBlock;
	}

	/**
	 * FG alle : Typ: alle Id: alle Name: Abruf DE-Block (Senden). Umsetzung erfolg in Attributgruppe/Aspekt
	 * <code>atg.tlsGloDeBlockAbruf/asp.tlsAbruf</code><br> TLS-Datenbeschreibung:
	 * <pre>
	 * Position         Bezeichnung                             Erläuterung
	 * Byte 1           Länge DE-Block                          Länge des folgenden DE-Blocks [ 3 ]
	 * Byte 2           Daten-Endgeräte-Kanal (DE)              [ 1..254, 255 ]
	 * Byte 3           Typ der DE-Daten                        [ alle ]
	 * Byte 4-n         DE-Daten                                [ 1..255 ]
	 *
	 * </pre>
	 *
	 * @param result Der vom Datenverteiler empgangene Datensatz, der in einen De-Block konvertiert wird.
	 *
	 * @return De-Block mit den konvertierten Daten.
	 */
	public DeBlock Fg255Typ255Id255AtgTlsGloDeBlockSendenAspTlsVorgabe(ResultData result) {
		DeBlock deBlock = null;

		// Datenblock beseht aus folgenden Inhalten:
		int _JobNummer_;//TBD: Jobnummer gehört nicht zum DE-Block sondern zum Einzeltelegramm. Umsetzung ????
		int _ID_;
		int _Typ_;
		int _Daten_;

		Data data = result.getData();// Datensatz holen

		_JobNummer_ = data.getUnscaledValue("JobNummer").intValue();
		_ID_ = data.getUnscaledValue("ID").intValue();
		_Typ_ = data.getUnscaledValue("Typ").intValue();

		int count = data.getArray("Daten").getLength();
		TlsData deApplicationData = new TlsData(count);

		for (int i = 0; i < count; i++) {
			_Daten_ = data.getArray("Daten").getItem(i).asUnscaledValue().intValue();
			deApplicationData.writeIntAsByte(_Daten_);
		}

		// DeBlock erzeugen
		try {
			deBlock = new DeBlock(deApplicationData, _de.getEak().getSm().getOsi7Adress(), _de.getFg(), _Typ_, _ID_, _de.getDeChannel());
		}
		catch (IllegalOsi7TelegramStructureException e) {
			debug.warning("Fehler beim Erzeugen eines De-Blocks zum Versand: " + e);
		}
		return deBlock;
	}
}