/*
 * Copyright (c) 2015 by inovat, innovative systeme - verkehr - tunnel - technik, Dipl.-Ing. H. C. Kniss
 *
 * This file is part of de.bsvrz.kex.tls.osi7.conversion.nw.uz.loehne.Fg001Duerr
 *
 * de.bsvrz.kex.tls.osi7.conversion.nw.uz.loehne.Fg001Duerr is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  de.bsvrz.kex.tls.osi7.conversion.nw.uz.loehne.Fg001Duerr is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with de.bsvrz.kex.tls.osi7.conversion.nw.uz.loehne.Fg001Duerr.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Diese Datei ist Teil von de.bsvrz.kex.tls.osi7.conversion.nw.uz.loehne.Fg001Duerr.
 *
 *  de.bsvrz.kex.tls.osi7.conversion.nw.uz.loehne.Fg001Duerr ist Freie Software: Sie können es unter den Bedingungen
 *  der GNU General Public License, wie von der Free Software Foundation,
 *  Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
 *  veröffentlichten Version, weiterverbreiten und/oder modifizieren.
 *
 *  de.bsvrz.kex.tls.osi7.conversion.nw.uz.loehne.Fg001Duerr wird in der Hoffnung, dass es nützlich sein wird, aber
 *  OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 *  Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
 *  Siehe die GNU General Public License für weitere Details.
 *
 *  Sie sollten eine Kopie der GNU General Public License zusammen mit diesem
 *  Programm erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 *
 * Contact Information:
 * inovat, Dipl.-Ing. H. C. Kniss
 * An der Krautwiese 37
 * D-53783 Eitorf
 * +49 (0)2243 8464 193
 * info@inovat.de
 * www.inovat.de
 */

package de.bsvrz.kex.tls.osi7.conversion.nw.uz.loehne;

import de.bsvrz.dav.daf.main.*;
import de.bsvrz.dav.daf.main.config.IntegerValueState;
import de.bsvrz.kex.tls.osi7.*;
import de.bsvrz.sys.funclib.debug.Debug;

import java.util.ArrayList;
import java.util.List;

/**
 * Umsetungsmodul für die FG 1, Standard (alle Typen der FG 1 gemäß TLS 2002). Wird standardmäßig für alle FG 1 De
 * verwendet, bei denen keine spezielle Umsezungsklasse in der Konfiguration angegeben wurde.
 *
 * @author inovat, innovative systeme - verkehr - tunnel - technik
 * @author Dipl.-Ing. Hans Christian Kniß (HCK)
 * @version $Revision$ / $Date$ / ($Author$)
 */

public class Fg001Duerr extends FgAllDuerr {
// ------------------------------------------------------------------------
// VARIABLENDEKLARATIONEN
// ------------------------------------------------------------------------

// Sonstige ---------------------------------------------------------------
	/** DebugLogger für Debug-Ausgaben */
	private static final Debug debug = Debug.getLogger();
// Private ----------------------------------------------------------------
	/**
	 * Tabelle, in der das Verhalten beim Empfang von Attributgruppen eingetragen wird. Jede Tabellenzeile hat folgenden
	 * Aufbau:
	 * <pre>
	 * "Attributgruppe (Pid)", "Aspekt (Pid)", "Fg", "Typ", "Id", "ObjektTyp"
	 * z.B.
	 * "atg.tlsGloKanalSteuerung", "asp.parameterSoll", "1", "29", "2",
	 * "typ.deLve"
	 * </pre>
	 * Die Auswertung erfolgt dabei wie folgt: <ul> <li> Für jede Attributguppen/Aspekt Kombination erfolgt eine
	 * Empfangsanmeldung als Empfänger.</li> <li> In der Methode {@link #update} wird für jede zur
	 * Attributgruppen/Aspket-Kombination passende Kombination eine Methode aufgerufen, deren Namen aus dem Tabelleneintrag
	 * wie folgt gebildet wird (hier für den Beispieleintrag):<br> <code>Fg001Typ029Id002AtgTlsGloKanalSteuerungAspParameterSoll(ResultData)</code><br>
	 * Dabei enthält <code>ResultData</code> die Daten der zu verarbeitenden ATG/ASP-Kombination.</li> <li> Für jede
	 * Attributguppen/Aspekt Kombination erfolgt eine Empfangsanmeldung als Empfänger. Sind mehrere passende Eintrage
	 * vorhanden, werden entsprechend mehrere Methodenaufrufe durchgeführt. Dadurch können mehrere ATG/ASP-Kombinationen in
	 * einen De-Block abgebildet werden (z.B. protokollnahe TLS-Attributgruppe und Attributgruppe eines Fachmodells):</li>
	 * <li> Ein Eintrag von "255" entspricht dabei einer "Wildcard" (passt immer). </ul>
	 */
	private final Object[][] davToTls = new Object[][]{
			// Attributgruppe, Aspekt, Fg, Typ, Id, ObjektTyp
			{"atg.tlsGloDeBlockAbruf", "asp.tlsAbruf", "255", "255", "255", "typ.deLve"},
			{"atg.tlsGloDeBlockSenden", "asp.tlsVorgabe", "255", "255", "255", "typ.deLve"},
			{"atg.tlsGloKanalSteuerung", "asp.parameterSoll", "255", "29", "2", "typ.deLve"},
			{"atg.tlsGloGeographischeKenndaten", "asp.tlsVorgabe", "255", "36", "3", "typ.deLve"},
			{"atg.tlsLveAbrufPufferInhalt", "asp.tlsAbruf", "1", "20", "2", "typ.deLve"},

			{"atg.tlsLveBetriebsParameter", "asp.parameterSoll", "1", "32", "3", "typ.deLve"},
	};

	/**
	 * Tabelle, in der das Verhalten beim Empfang von TLS De-Blöcken eingetragen wird. Jede Tabellenzeile hat folgenden
	 * Aufbau:
	 * <pre>
	 * "Attributgruppe (Pid)", "Aspekt (Pid)", "Fg", "Typ", "Id", "ObjektTyp"
	 * z.B.
	 * "atg.tlsGloKanalSteuerung", "asp.tlsAntwort", "1", "29", "2", "typ.deLve"
	 * </pre>
	 * Die Auswertung erfolgt dabei wie folgt: <ul> <li> Für jede Attributguppen/Aspekt Kombination erfolgt eine
	 * Sendeanmeldung als Quelle.</li> <li> In der Methode {@link #processDeBlock} wird für jede zum De-Block passende
	 * Fg/Id/Typ-Kombination eine Methode aufgerufen, deren Namen aus dem Tabelleneintrag wie folgt gebildet wird (hier für
	 * den Beispieleintrag):<br> <code>Fg001Typ029Id002AtgTlsGloKanalSteuerungAspTlsAntwort(DeBlock, Data,
	 * DataDescription)</code><br> Dabei enthält der <code>deBlock</code> die Daten des zu verarbeitenden De-Blocks und
	 * <code>data</code> ein Datenobjekt der passenden Attributgruppen/Aspekt-Kombination, welchem direkt die Inhalte des
	 * De-Blocks zugewiesen werden können.<code>DataDescription</code> enthält die passende {@link
	 * DataDescription} für die zu sendenden Daten.</li> <li> Für jede Attributguppen/Aspekt
	 * Kombination erfolgt eine Sendeanmeldung als Quelle. Sind mehrere passende Eintrage vorhanden, werden entsprechend
	 * mehrere Methodenaufrufe durchgeführt. Dadurch kann ein De-Block mehrfach ausgewert und an verschiedene
	 * Attributgruppen weitergerleitet werden (z.B. protokollnahe TLS-Attributgruppe und Attributgruppe eine
	 * Fachmodells):</li> <li> Ein Eintrag von "255" entspricht dabei einer "Wildcard" (passt immer). </ul>
	 */
	private final Object[][] tlsToDav = new Object[][]{
			// Attributgruppe, Aspekt, Fg, Typ, Id, ObjektTyp
			{"atg.tlsGloDeFehler", "asp.tlsAntwort", "1", "1", "1", "typ.deLve"},
			{"atg.tlsGloKanalSteuerung", "asp.parameterIst", "255", "29", "2", "typ.deLve"},
			{"atg.tlsGloGeographischeKenndaten", "asp.tlsZustand", "255", "36", "3", "typ.deLve"},
			{"atg.tlsLveErgänzendeDeFehlerMeldung", "asp.tlsAntwort", "1", "14", "1", "typ.deLve"},
			{"atg.tlsLveNegativeQuittung", "asp.tlsAntwort", "1", "16", "2", "typ.deLve"},
			{"atg.tlsLveErgebnisMeldungVersion0Bis4", "asp.tlsAntwort", "1", "49", "4", "typ.deLve"},
			{"atg.tlsLveErgebnisMeldungVersion0Bis4", "asp.tlsAntwort", "1", "50", "4", "typ.deLve"},
			{"atg.tlsLveErgebnisMeldungVersion0Bis4", "asp.tlsAntwort", "1", "51", "4", "typ.deLve"},
			{"atg.tlsLveErgebnisMeldungVersion0Bis4", "asp.tlsAntwort", "1", "52", "4", "typ.deLve"},
			{"atg.tlsLveErgebnisMeldungVersion0Bis4", "asp.tlsAntwort", "1", "53", "4", "typ.deLve"},
			{"atg.tlsLveErgebnisMeldungVersion0Bis4", "asp.tlsAntwort", "1", "113", "4", "typ.deLve"},
			{"atg.tlsLveErgebnisMeldungVersion0Bis4", "asp.tlsAntwort", "1", "114", "4", "typ.deLve"},
			{"atg.tlsLveErgebnisMeldungVersion0Bis4", "asp.tlsAntwort", "1", "115", "4", "typ.deLve"},
			{"atg.tlsLveErgebnisMeldungVersion0Bis4", "asp.tlsAntwort", "1", "116", "4", "typ.deLve"},
			{"atg.tlsLveErgebnisMeldungVersion0Bis4", "asp.tlsAntwort", "1", "117", "4", "typ.deLve"},

			{"atg.tlsLveBetriebsParameter", "asp.parameterIst", "1", "32", "3", "typ.deLve"},

			{"atg.verkehrsDatenKurzZeitIntervall", "asp.externeErfassung", "1", "49", "4", "typ.fahrStreifen"},
			{"atg.verkehrsDatenKurzZeitIntervall", "asp.externeErfassung", "1", "50", "4", "typ.fahrStreifen"},
			{"atg.verkehrsDatenKurzZeitIntervall", "asp.externeErfassung", "1", "51", "4", "typ.fahrStreifen"},
			{"atg.verkehrsDatenKurzZeitIntervall", "asp.externeErfassung", "1", "52", "4", "typ.fahrStreifen"},
			{"atg.verkehrsDatenKurzZeitIntervall", "asp.externeErfassung", "1", "53", "4", "typ.fahrStreifen"},
			{"atg.verkehrsDatenKurzZeitIntervall", "asp.externeErfassung", "1", "113", "4", "typ.fahrStreifen"},
			{"atg.verkehrsDatenKurzZeitIntervall", "asp.externeErfassung", "1", "114", "4", "typ.fahrStreifen"},
			{"atg.verkehrsDatenKurzZeitIntervall", "asp.externeErfassung", "1", "115", "4", "typ.fahrStreifen"},
			{"atg.verkehrsDatenKurzZeitIntervall", "asp.externeErfassung", "1", "116", "4", "typ.fahrStreifen"},
			{"atg.verkehrsDatenKurzZeitIntervall", "asp.externeErfassung", "1", "117", "4", "typ.fahrStreifen"},

	};

	/**
	 * Art der Mittelwertbildung, die bei diesem De eingestellt ist. Wird sowohl beim Setzen der Betriebs- parameter als
	 * auch beim Empfang der Rückmeldung (sofern diese erfolgt) angepasst. -1 = unbekannt 0 = gleitende Mittelwertbildung 1
	 * = arithmetische Mittelwertbildung
	 */
	private int _kindOfAverage = -1;// initial auf undefiniert

// ------------------------------------------------------------------------
// KONSTRUKTOREN  (und vom Konstruktor verwende Methoden)
// ------------------------------------------------------------------------

	/**
	 * Erzeugt ein Objekt vom Typ Fg001Default
	 *
	 * @param daf DaV
	 * @param eak Eak
	 * @param de De
	 */
	public Fg001Duerr(ClientDavInterface daf, Eak eak, De de) {
		super(daf, eak, de);
		subscribeAllData(tlsToDav, davToTls);
	}

// ------------------------------------------------------------------------
// GETTER / SETTER METHODEN
// ------------------------------------------------------------------------

	//  setzten
	public int getKindOfAverage() {
		return _kindOfAverage;
	}

// ------------------------------------------------------------------------
// SONSTIGE METHODEN
// ------------------------------------------------------------------------

	/**
	 * FG 1 : Typ: 1 Id: 1 Name: DE-Fehler (Antwort). Ergänzt globale Behandlung De-Fehler in {@link
	 * #Fg255Typ001Id001AtgTlsGloDeFehlerAspTlsAntwort}. Beim Empfang einer De-Gutmeldung wird die De initialisiert, indem
	 * <ul> <li>LVE-kanalsteuerung {@link #Fg255Typ029Id002AtgTlsGloKanalSteuerungAspParameterSoll}</li>
	 * <li>LVE-betriebsparameter{@link #Fg001Typ032Id003AtgTlsLveBetriebsParameterAspParameterSoll}</li> </ul> versendet
	 * werden.
	 *
	 * @param deBlock         Der zu bearbeitende De-Block.
	 * @param data            Der passende Datensatz für die Ziel Attributgruppe/Aspekt-Kombination.
	 * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
	 *
	 * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
	 */
	public ResultData Fg001Typ001Id001AtgTlsGloDeFehlerAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
		ResultData resultData = super.Fg255Typ001Id001AtgTlsGloDeFehlerAspTlsAntwort(deBlock, data, dataDescription);

		IntegerValueState DEFehlerStatus = data.getUnscaledValue("DEFehlerStatus").getState();
		if (DEFehlerStatus != null) {
			debug.fine("DEFehlerStatus: " + DEFehlerStatus.getName());
			if (DEFehlerStatus.getName().equals("ok")) {
				sendInitialDeBlock(deBlock.getKey(), 29, 2);// LVE-Kanalsteuerung senden
				sendInitialDeBlock(deBlock.getKey(), 32, 3);// LVE-Betriebsparameter senden
			}
			else {
				List deList = new ArrayList();
				deList.add(_de);
				TlsApplicationLayer.writeNoDataToUnreachableDe(deList, _eakConversionModul.getTimeStamp(), -1, "atg.tlsGloDeFehler");
			}
		}

		return resultData;
	}

	/**
	 * FG 1 : Typ: 14 Id: 1 Name: Ergänzende DE-Fehlermeldung (Antwort) TLS-Datenbeschreibung:
	 * <pre>
	 * Position         Bezeichnung                             Erläuterung
	 * Byte 1           Länge DE-Block                          [ 9..29 ]
	 * Byte 2           Daten-Endgeräte-Kanal                   [ 1..254 ]
	 * Byte 3           Typ der DE-Daten                        [ 14 ]
	 * Byte 4           Herstellercode                          (s. Anh. 7, Kap.
	 * 2)
	 * Byte 5           Anzahl TLS  Fehlerbytes                [ 4 ]
	 * Byte 6           TLS - Fehlerbyte 1                      Codierung siehe
	 * unten
	 * Byte 7           TLS - Fehlerbyte 2                      Codierung siehe
	 * unten
	 * Byte 8           TLS - Fehlerbyte 3                      Codierung siehe
	 * unten
	 * Byte 9           TLS - Fehlerbyte 4                      Codierung siehe
	 * unten
	 * Byte 10          Anzahl Hersteller Fehlerbytes           [ 0..20 ]
	 * Byte 11          Hersteller-Fehlerbyte 1                 herstellerspez.
	 * Codierung
	 * Byte..           Hersteller-Fehlerbyte n
	 * </pre>
	 * Inhalt TLS-Fehlerbytes:<br> Die Fehlerbytes sind bitweise codiert. Jedes Bit definiert einen bestimmten Fehler,
	 * wobei das Bit gleich 1 gesetzt ist.<br> Es können auch mehrere Bit gleichzeitig gesetzt werden. Der Inhalt der
	 * Fehlerbytes ist optional. D.h. bei einer Realisierung dieses Telegrammtyps müssen vom Hersteller nicht alle in der
	 * Tabelle definierten Fehlercodes erfüllt werden. Für spätere Definitionen reservierte Bits bzw. nicht verwendete Bits
	 * sind gleich Null zu setzen.
	 * <pre>
	 *         Bit 7      Bit 6       Bit 5       Bit 4          Bit 3         Bit
	 * 2       Bit 1       Bit 0
	 * Byte 1  Reserve    Reserve     RNR am      Lokalbus       Reserve
	 * Reserve     Reserve     Teilstörung
	 *                                Lokalbus1   ausgefallen
	 * Byte 2  Abgleich   Abgleich    Schleife 2  Schleife 1     Auswertung
	 * Detektor    Subbus      Subbus
	 *         Schleife   Schleife    defekt 2)   defekt 2)      gestört
	 * gestört     RNR         ausgefallen
	 *         2 2)       1 2)
	 * Byte 3  Reserve    Reserve     Reserve     Reserve        Reserve
	 * Reserve     Reserve     Reserve
	 * Byte 4  Reserve    Reserve     Reserve     Reserve        Reserve
	 * Reserve     Reserve     Reserve
	 * </pre>
	 * 1) nur in Verbindung mit Stö/SM=1 im Byte 4 der DE-Block-Struktur im Typ 1<br> 2) Schleife 1 und Schleife 2 sind die
	 * 1. bzw. 2. Schleife einer Doppelschleife desselben Fahrstreifens, gezählt in Fahrtrichtung.
	 *
	 * @param deBlock         Der zu bearbeitende De-Block.
	 * @param data            Der passende Datensatz für die Ziel Attributgruppe/Aspekt-Kombination.
	 * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
	 *
	 * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
	 */
	public ResultData Fg001Typ014Id001AtgTlsLveErgänzendeDeFehlerMeldungAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
		ResultData resultData = null;

		// Herstellercode-Byte aus Telegramm ---------------------------------------------------------------------------
		int _Herstellercode_ = deBlock.getDeData().readUnsignedByteAsInt();
		data.getUnscaledValue("Hersteller").set(_Herstellercode_);

		// Fehler-Bytes aus Telegramm -----------------------------------------------------------------------------------
		deBlock.getDeData().readUnsignedByteAsInt();// Anzahl der folgenden Bytes überlesen, da immer == 4 !
		int _TlsFehler1_ = deBlock.getDeData().readUnsignedByteAsInt();
		int _TlsFehler2_ = deBlock.getDeData().readUnsignedByteAsInt();
		int _TlsFehler3_ = deBlock.getDeData().readUnsignedByteAsInt();
		int _TlsFehler4_ = deBlock.getDeData().readUnsignedByteAsInt();

		Data.NumberArray _TlsFehler_ = data.getItem("FehlercodesTLS").getUnscaledArray("Fehler");
		_TlsFehler_.setLength(4);
		_TlsFehler_.getValue(0).set(_TlsFehler1_);
		_TlsFehler_.getValue(1).set(_TlsFehler2_);
		_TlsFehler_.getValue(2).set(_TlsFehler3_);
		_TlsFehler_.getValue(3).set(_TlsFehler4_);

		// Hersteller Fehler-Bytes aus Telegramm -----------------------------------------------------------------------
		int count = deBlock.getDeData().readUnsignedByteAsInt();
		int[] _HerstellerFehler_ = new int[count];
		for (int i = 0; i < count; i++) {
			_HerstellerFehler_[i] = deBlock.getDeData().readUnsignedByteAsInt();
		}

		Data.NumberArray _Fehler_ = data.getItem("FehlercodesHersteller").getUnscaledArray("Fehler");
		_Fehler_.setLength(count);
		for (int i = 0; i < count; i++) {
			_Fehler_.getValue(i).set(_HerstellerFehler_[i]);
		}

		//Ergebnisdatensatz erzeugen...
		resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);
		return resultData;
	}

	/**
	 * FG 1 : Typ: 16 Id: 2 Name: Negative Quittung (Antwort) TLS-Datenbeschreibung:<br>
	 * <pre>
	 * Position         Bezeichnung                             Erläuterung
	 * Byte 1           Länge DE-Block                          Länge des folgenden
	 * DE-Blocks [ 4 ]
	 * Byte 2           Daten-Endgeräte-Kanal                   [ 1..254 ]
	 * Byte 3           Typ der DE-Daten                        [ 16 ]
	 * Byte 4           Fehlerursache siehe unten
	 * Byte 5           Herstellercode                          siehe Liste in
	 * Anhang 7, Kap. 2
	 * </pre>
	 * Inhalt Byte: Fehlerursache unter FG 1:<br>
	 * <pre>
	 *         0:     sonstige Fehlerursache
	 *         1:     unbekannte oder nicht auswertbare ID
	 *         2:     unbekannter oder nicht auswertbarer Typ
	 *         3:     Version nicht verfügbar
	 *     4..11:     reserviert für spätere Definitionen
	 *        12:     Pufferinhalt nicht verfügbar
	 *   13..127:     reserviert für spätere Definitionen
	 *  128..255:     frei für herstellerdefinierte Ursachencodes
	 * </pre>
	 * Inhalt Byte Herstellercode:<br> Der Herstellercode ist ein für jeden Hersteller eindeutig zugeordneter Code, der der
	 * Zentrale die Zuordnung der Nachrichten erleichtern soll.
	 *
	 * @param deBlock         Der zu bearbeitende De-Block.
	 * @param data            Der passende Datensatz für die Ziel Attributgruppe/Aspekt-Kombination.
	 * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
	 *
	 * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
	 */
	public ResultData Fg001Typ016Id002AtgTlsLveNegativeQuittungAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
		return super.Fg255Typ016Id002AtgTlsGloNegativeQuittungAspTlsAntwort(deBlock, data, dataDescription);
	}

	/**
	 * FG 1 : Typ: 32 Id: 3 Name: LVE-Betriebsparameter (Abruf / Antwort) TLS-Datenbeschreibung:
	 * <pre>
	 * Position         Bezeichnung                                Erläuterung
	 * Byte 1           Länge DE-Block                             Länge des
	 * folgenden DE-Blocks [ 10 ]
	 * Byte 2           Daten-Endgeräte-Kanal                      [1..254, 255]
	 * Byte 3           Typ der DE-Daten                           [ 32 ]
	 * Byte 4           Datenversion/Kurzzeitdaten                 [ 0.. 4, 255 =
	 * aus ]
	 * Byte 5           Erfassungsintervalldauer/Kurzzeitdaten     [
	 * 1,2,4,8,12...240, Einheit: 15 s ]
	 * Byte 6           Datenversion/Langzeitdaten                 [ 10.. 23, 255 =
	 * aus ]
	 * Byte 7           Erfassungsintervalldauer/Langzeitdaten     siehe unten
	 * Byte 8           Glättungsparameter                         Alpha 1 Alpha 1
	 * = (Byteinhalt + 1) / 256
	 * Byte 9           Glättungsparameter                         Alpha 2 Alpha 2
	 * = (Byteinhalt + 1) / 256
	 * Byte 10          Längengrenzwert
	 * Lkw/Pkw-Unterscheidung 4m+[0..254cm]
	 * Byte 11          Art / Startwert für gl. Mittelwertbildung  siehe unten
	 * </pre>
	 * Inhalt Byte: Datenversion / Kurzzeitdaten<br> Mit Hilfe dieser Bytes wird die Datenversion ausgewählt, mit der die
	 * zyklisch erfassten Verkehrsdaten übertragen werden sollen. Bei Geräten, die die Version 4 nicht unterstützen, wird
	 * eine 4 in diesem Byte mit einer negativen Quittung (Fehlerursache 3) beantwortet. Wenn auf die Versionen 4
	 * umgeschaltet wird, ohne dass zuvor mit dem DEBlock Typ 37 eine entsprechende Parametrierung der Grenzen der
	 * Geschwindigkeitsklassen vorgenommen wurde, dann werden die dort beschriebenen Defaultwerte für die Grenzen der
	 * Geschwindigkeitsklassen verwendet.<br>
	 * <p>
	 * Inhalt Byte: Erfassungsintervalldauer / Kurzzeitdaten:<br> Das Byte enthält den Wert der Erfassungsintervalldauer in
	 * 15-Sekunden-Einheiten. Es sind nur Werte zugelassen, die ein ganzzahliges Vielfaches haben, welches 60 Minuten
	 * ergibt (15s, 30s, 1, 2, 3, 4, 5, 6, 10, 12, 15, 20, 30, 60 Minuten).
	 * <p>
	 * Inhalt Byte: Datenversion / Langzeitdaten<br> Mit Hilfe dieser Bytes wird die Datenversion ausgewählt, mit der die
	 * zyklisch erfassten Verkehrsdaten übertragen werden sollen. Geräte, von denen eine nicht implementierte Version
	 * gefordert wird, antworten mit einer negativen Quittung (Fehlerursache 3). Wenn auf die Versionen 17, 18, 22 oder 23
	 * umgeschaltet wird, ohne dass zuvor mit dem DE-Block Typ 38 eine entsprechende Parametrierung der Grenzen der
	 * Geschwindigkeitsklassen vorgenommen wurde, dann werden die dort beschriebenen Defaultwerte für die Grenzen der
	 * Geschwindigkeitsklassen verwendet.<br>
	 * <p>
	 * Inhalt Byte: Erfassungsintervalldauer / Langzeitdaten<br> Das Byte enthält den Wert der Erfassungsintervalldauer in
	 * Stunden-Einheiten:<br>
	 * <pre>
	 * Bit7 = 1:    Einheit = Stunden
	 * Bit0..6:     Zeitwert in Stunden [1..24]
	 * </pre>
	 * Es sind nur Stundenwerte zugelassen, die ganzzahlige Teiler oder ganzzahlige Mehrfache von 24 Stunden sind (1, 2, 3,
	 * 4, 6, 8, 12, 24 Stunden).<br>
	 * <p>
	 * Inhalt Byte: Art / Startwert für gleitende Mittelwertbildung:<br> Die gleitende Mittelwertbildung wird nur bei
	 * Kurzzeitdaten angewandt. Bei Langzeitdaten wird der arithmetische Mittelwert verwendet.<br>
	 * <pre>
	 * Bit 0..6:    Startwert der Geschwindigkeit für gleitende Mittelwertbildung
	 * [0..127km/h]
	 * Bit 7:       Art der Geschwindigkeits-Mittelwertbildung
	 *              0=gleitende Mittelwertbildung
	 *              1=arithmetische Mittelwertbildung
	 * Wichtig:     Bei Auswahl der Version 3 oder 4 müssen immer die
	 * Glättungsparameter und Startwerte angegeben werden,
	 *              da in diesen Telegrammversionen zusätzlich der gleitende
	 * Mittelwert für Steuerungszwecke unabhängig
	 *              von der Einstellung in Bit 7 übertragen wird.
	 * </pre>
	 * Inhalt Byte: Längengrenzwert<br> Bei E/A-Konzentratoren ohne Längenauswertung wird bei der Zuweisung das Byte
	 * ignoriert. In der Antwort dieses E/A-Konzentrators wird es auf 255 gesetzt.<br>
	 *
	 * @param deBlock         Der zu bearbeitende De-Block.
	 * @param data            Der passende Datensatz für die Ziel Attributgruppe/Aspekt-Kombination.
	 * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
	 *
	 * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
	 */
	public ResultData Fg001Typ032Id003AtgTlsLveBetriebsParameterAspParameterIst(DeBlock deBlock, Data data, DataDescription dataDescription) {
		ResultData resultData = null;

		// VersionKurz-Byte aus Telegramm ------------------------------------------------------------------------------
		int _VersionKurz_ = deBlock.getDeData().readUnsignedByteAsInt();

		switch (_VersionKurz_) {
			case 0:
			case 1:
			case 2:
			case 3:
			case 4:
			case 255:
				data.getUnscaledValue("VersionKurzZeitDaten").set(_VersionKurz_);
				break;
			default:
				debug.warning("VersionKurzZeitDaten ungültig : " + _VersionKurz_ + Debug.NEWLINE +
				              "De-Block (Osi7Adresse-DeNummer-FG-Typ-Id): " + deBlock.getLongKey() + " verworfen:" + Debug.NEWLINE +
				              deBlock.toString());
				return resultData;
		}

		// Intervalldaten Kurzzeit-Byte aus Telegramm ------------------------------------------------------------------
		int _IntervallKurz_ = deBlock.getDeData().readUnsignedByteAsInt();

		switch (_IntervallKurz_) {
			case 1:
			case 2:
			case 4:
			case 8:
			case 12:
			case 16:
			case 20:
			case 24:
			case 40:
			case 48:
			case 60:
			case 80:
			case 120:
			case 240:
				data.getUnscaledValue("IntervallDauerKurzZeitDaten").set(_IntervallKurz_);
				break;
			default:
				debug.warning("IntervallDauerKurzZeitDaten ungültig : " + _IntervallKurz_ + Debug.NEWLINE +
				              "De-Block (Osi7Adresse-DeNummer-FG-Typ-Id): " + deBlock.getLongKey() + " verworfen:" + Debug.NEWLINE +
				              deBlock.toString());
				return resultData;
		}

		// VersionLang-Byte aus Telegramm ------------------------------------------------------------------------------
		int _VersionLang_ = deBlock.getDeData().readUnsignedByteAsInt();
		switch (_VersionLang_) {
			case 10:
			case 11:
			case 12:
			case 13:
			case 14:
			case 15:
			case 16:
			case 17:
			case 18:
			case 19:
			case 20:
			case 21:
			case 22:
			case 23:
			case 255:
				data.getUnscaledValue("VersionLangZeitDaten").set(_VersionLang_);
				break;
			default:
				debug.warning("VersionLangZeitDaten ungültig : " + _VersionLang_ + Debug.NEWLINE +
				              "De-Block (Osi7Adresse-DeNummer-FG-Typ-Id): " + deBlock.getLongKey() + " verworfen:" + Debug.NEWLINE +
				              deBlock.toString());
				return resultData;
		}

		// Întervalldaten Langzeit-Byte aus Telegramm ------------------------------------------------------------------
		int _IntervallLang_ = deBlock.getDeData().readUnsignedByteAsInt();

		switch (_IntervallLang_ & 0x7f) {
			case 1:
			case 2:
			case 3:
			case 4:
			case 6:
			case 8:
			case 12:
			case 24:
				data.getUnscaledValue("IntervallDauerLangZeit").set(_IntervallLang_);
				break;
			default:
				debug.warning("IntervallDauerLangZeit ungültig : " + _IntervallLang_ + Debug.NEWLINE +
				              "De-Block (Osi7Adresse-DeNummer-FG-Typ-Id): " + deBlock.getLongKey() + " verworfen:" + Debug.NEWLINE +
				              deBlock.toString());
				return resultData;
		}

		// Alpha 1-Byte aus Telegramm ----------------------------------------------------------------------------------
		int _alpha1_ = deBlock.getDeData().readUnsignedByteAsInt();
		data.getUnscaledValue("alpha1").set(_alpha1_ + 1);

		// Alpha 2-Byte aus Telegramm ----------------------------------------------------------------------------------
		int _alpha2_ = deBlock.getDeData().readUnsignedByteAsInt();
		data.getUnscaledValue("alpha2").set(_alpha2_ + 1);

		// Längengrenzwert für LKW-Byte aus Telegramm ------------------------------------------------------------------
		int _LaengeLKW_ = deBlock.getDeData().readUnsignedByteAsInt();
		if (_LaengeLKW_ == 255) {
			data.getUnscaledValue("LängenGrenzWert").setText("kein Längengrenzwert");
		}
		else {
			data.getUnscaledValue("LängenGrenzWert").set(_LaengeLKW_ + 400);
		}

		// Startwert für Mittelwertbilung-Byte aus Telegramm -----------------------------------------------------------
		int _MittelWert_ = deBlock.getDeData().readUnsignedByteAsInt();

		int _MittelWertBildung_ = _MittelWert_ & 0x80;
		int _MittelWertStart_ = _MittelWert_ & 0x7f;

		if (_MittelWertBildung_ == 0) {
			data.getUnscaledValue("ArtMittelWertBildung").setText("gleitende Mittelwertbildung");
			_kindOfAverage = 0;
		}
		else {
			data.getUnscaledValue("ArtMittelWertBildung").setText("arithmetische Mittelwertbildung");
			_kindOfAverage = 1;
		}

		data.getUnscaledValue("StartMittelWertBildung").set(_MittelWertStart_);

		//Ergebnisdatensatz erzeugen...
		resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);
		return resultData;
	}

	/**
	 * FG 1 : Typ: 32 Id: 3 Name: LVE-Betriebsparameter (Abruf / Antwort) TLS-Datenbeschreibung:
	 * <pre>
	 * Position         Bezeichnung                                Erläuterung
	 * Byte 1           Länge DE-Block                             Länge des
	 * folgenden DE-Blocks [ 10 ]
	 * Byte 2           Daten-Endgeräte-Kanal                      [1..254, 255]
	 * Byte 3           Typ der DE-Daten                           [ 32 ]
	 * Byte 4           Datenversion/Kurzzeitdaten                 [ 0.. 4, 255 =
	 * aus ]
	 * Byte 5           Erfassungsintervalldauer/Kurzzeitdaten     [
	 * 1,2,4,8,12...240, Einheit: 15 s ]
	 * Byte 6           Datenversion/Langzeitdaten                 [ 10.. 23, 255 =
	 * aus ]
	 * Byte 7           Erfassungsintervalldauer/Langzeitdaten     siehe unten
	 * Byte 8           Glättungsparameter                         Alpha 1 Alpha 1
	 * = (Byteinhalt + 1) / 256
	 * Byte 9           Glättungsparameter                         Alpha 2 Alpha 2
	 * = (Byteinhalt + 1) / 256
	 * Byte 10          Längengrenzwert
	 * Lkw/Pkw-Unterscheidung 4m+[0..254cm]
	 * Byte 11          Art / Startwert für gl. Mittelwertbildung  siehe unten
	 * </pre>
	 * Inhalt Byte: Datenversion / Kurzzeitdaten<br> Mit Hilfe dieser Bytes wird die Datenversion ausgewählt, mit der die
	 * zyklisch erfassten Verkehrsdaten übertragen werden sollen. Bei Geräten, die die Version 4 nicht unterstützen, wird
	 * eine 4 in diesem Byte mit einer negativen Quittung (Fehlerursache 3) beantwortet. Wenn auf die Versionen 4
	 * umgeschaltet wird, ohne dass zuvor mit dem DEBlock Typ 37 eine entsprechende Parametrierung der Grenzen der
	 * Geschwindigkeitsklassen vorgenommen wurde, dann werden die dort beschriebenen Defaultwerte für die Grenzen der
	 * Geschwindigkeitsklassen verwendet.<br>
	 * <p>
	 * Inhalt Byte: Erfassungsintervalldauer / Kurzzeitdaten:<br> Das Byte enthält den Wert der Erfassungsintervalldauer in
	 * 15-Sekunden-Einheiten. Es sind nur Werte zugelassen, die ein ganzzahliges Vielfaches haben, welches 60 Minuten
	 * ergibt (15s, 30s, 1, 2, 3, 4, 5, 6, 10, 12, 15, 20, 30, 60 Minuten).
	 * <p>
	 * Inhalt Byte: Datenversion / Langzeitdaten<br> Mit Hilfe dieser Bytes wird die Datenversion ausgewählt, mit der die
	 * zyklisch erfassten Verkehrsdaten übertragen werden sollen. Geräte, von denen eine nicht implementierte Version
	 * gefordert wird, antworten mit einer negativen Quittung (Fehlerursache 3). Wenn auf die Versionen 17, 18, 22 oder 23
	 * umgeschaltet wird, ohne dass zuvor mit dem DE-Block Typ 38 eine entsprechende Parametrierung der Grenzen der
	 * Geschwindigkeitsklassen vorgenommen wurde, dann werden die dort beschriebenen Defaultwerte für die Grenzen der
	 * Geschwindigkeitsklassen verwendet.<br>
	 * <p>
	 * Inhalt Byte: Erfassungsintervalldauer / Langzeitdaten<br> Das Byte enthält den Wert der Erfassungsintervalldauer in
	 * Stunden-Einheiten:<br>
	 * <pre>
	 * Bit7 = 1:    Einheit = Stunden
	 * Bit0..6:     Zeitwert in Stunden [1..24]
	 * </pre>
	 * Es sind nur Stundenwerte zugelassen, die ganzzahlige Teiler oder ganzzahlige Mehrfache von 24 Stunden sind (1, 2, 3,
	 * 4, 6, 8, 12, 24 Stunden).<br>
	 * <p>
	 * Inhalt Byte: Art / Startwert für gleitende Mittelwertbildung:<br> Die gleitende Mittelwertbildung wird nur bei
	 * Kurzzeitdaten angewandt. Bei Langzeitdaten wird der arithmetische Mittelwert verwendet.<br>
	 * <pre>
	 * Bit 0..6:    Startwert der Geschwindigkeit für gleitende Mittelwertbildung
	 * [0..127km/h]
	 * Bit 7:       Art der Geschwindigkeits-Mittelwertbildung
	 *              0=gleitende Mittelwertbildung
	 *              1=arithmetische Mittelwertbildung
	 * Wichtig:     Bei Auswahl der Version 3 oder 4 müssen immer die
	 * Glättungsparameter und Startwerte angegeben werden,
	 *              da in diesen Telegrammversionen zusätzlich der gleitende
	 * Mittelwert für Steuerungszwecke unabhängig
	 *              von der Einstellung in Bit 7 übertragen wird.
	 * </pre>
	 * Inhalt Byte: Längengrenzwert<br> Bei E/A-Konzentratoren ohne Längenauswertung wird bei der Zuweisung das Byte
	 * ignoriert. In der Antwort dieses E/A-Konzentrators wird es auf 255 gesetzt.<br>
	 *
	 * @param result Der vom Datenverteiler empgangene Datensatz, der in einen De-Block konvertiert wird.
	 *
	 * @return De-Block mit den konvertierten Daten.
	 */
	public DeBlock Fg001Typ032Id003AtgTlsLveBetriebsParameterAspParameterSoll(ResultData result) {
		DeBlock deBlock = null;

		// Datenblock beseht aus folgenden Inhalten:
		int _VersionKurz_;
		int _IntervallKurz_;
		int _VersionLang_;
		int _IntervallLang_;
		int _alpha1_;
		int _alpha2_;
		int _LaengeLKW_;
		int _MittelWert_;
		//Hilfvariablen
		int _MittelWertBildung_;
		int _MittelWertStart_;

		// Umsetzung
		Data data = result.getData();// Datensatz holen

		_VersionKurz_ = data.getUnscaledValue("VersionKurzZeitDaten").intValue();
		_IntervallKurz_ = data.getUnscaledValue("IntervallDauerKurzZeitDaten").intValue();
		_VersionLang_ = data.getUnscaledValue("VersionLangZeitDaten").intValue();
		_IntervallLang_ = data.getUnscaledValue("IntervallDauerLangZeit").intValue();
		_alpha1_ = data.getUnscaledValue("alpha1").intValue() - 1;
		_alpha2_ = data.getUnscaledValue("alpha2").intValue() - 1;

		_LaengeLKW_ = data.getUnscaledValue("LängenGrenzWert").intValue();
		if (_LaengeLKW_ != 255) {
			_LaengeLKW_ -= 400;
		}

		_MittelWertBildung_ = data.getUnscaledValue("ArtMittelWertBildung").intValue();
		_kindOfAverage = _MittelWertBildung_;

		_MittelWertStart_ = data.getUnscaledValue("StartMittelWertBildung").intValue();

		_MittelWert_ = _MittelWertStart_ + (_MittelWertBildung_ << 7);

		// DeBlock als TlsData aufbauen...

		TlsData deApplicationData = new TlsData(8);
		deApplicationData.writeIntAsByte(_VersionKurz_);
		deApplicationData.writeIntAsByte(_IntervallKurz_);
		deApplicationData.writeIntAsByte(_VersionLang_);
		deApplicationData.writeIntAsByte(_IntervallLang_);
		deApplicationData.writeIntAsByte(_alpha1_);
		deApplicationData.writeIntAsByte(_alpha2_);
		deApplicationData.writeIntAsByte(_LaengeLKW_);
		deApplicationData.writeIntAsByte(_MittelWert_);

		// DeBlock erzeugen
		try {
			deBlock = new DeBlock(deApplicationData, _de.getEak().getSm().getOsi7Adress(), _de.getFg(), 32, 3, _de.getDeChannel());
		}
		catch (IllegalOsi7TelegramStructureException e) {
			debug.warning("Fehler beim Erzeugen eines De-Blocks zum Versand: " + e);
		}
		return deBlock;
	}

	public ResultData Fg001Typ049Id004AtgTlsLveErgebnisMeldungVersion0Bis4AspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
		return TlsLveErgebnisMeldungVersion0Bis4AspTlsAntwort(deBlock, data, dataDescription);
	}

	/**
	 * Verarbeitet alls Telegrammversionen der TLS Kurzeit Version 0-4 (Typ 49/113 -53/117).
	 *
	 * @param deBlock         Der zu bearbeitende De-Block.
	 * @param data            Der passende Datensatz für die Ziel Attributgruppe/Aspekt-Kombination.
	 * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
	 *
	 * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
	 */
	private ResultData TlsLveErgebnisMeldungVersion0Bis4AspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
		ResultData resultData = null;

		// Intervalllänge aus IntervalldatenHeader ---------------------------------------------------------------------
		data.getTimeValue("T").setSeconds(_eakConversionModul.getLveIntervalDuration());

		int _qKfz_;// qKfz-Wert aus Telegramm
		int _qLkwÄ_;// qLkw-Wert aus Telegramm
		int _vPkwÄ_;// qPkwÄ-Wert aus Telegramm
		int _vLkwÄ_;// vLkwÄ-Wert aus Telegramm
		int _tNetto_;// tNetto-Wert aus Telegramm
		int _b_;// Belegungs-Wert aus Telegramm
		int _sKfz_;// Streuung Kfz-Wert aus Telegramm
		int _vKfz_;// vKfz-Wert aus Telegramm

		int countQPkwÄGeschwKlasse = 0;
		int[] _qPkwÄGeschwKlasse_;// Geschwindigkeitsklasse qPkwÄ-Werte aus Telegramm
		int countQLkwÄGeschwKlasse = 0;
		int[] _qLkwÄGeschwKlasse_;// Geschwindigkeitsklasse qPkwÄ-Wertte aus Telegramm

		Data.NumberArray _KlassePkw_;
		Data.NumberArray _KlasseLkw_;

		// Werte jeweils auf Zustand "nicht ermittelbar" setzen.
		data.getUnscaledValue("qKfz").setText("nicht ermittelbar");
		data.getUnscaledValue("qLkwÄ").setText("nicht ermittelbar");
		data.getUnscaledValue("vPkwÄ").setText("nicht ermittelbar");
		data.getUnscaledValue("vLkwÄ").setText("nicht ermittelbar");
		data.getUnscaledValue("tNetto").setText("nicht ermittelbar");
		data.getUnscaledValue("b").setText("nicht ermittelbar");
		data.getUnscaledValue("sKfz").setText("nicht ermittelbar");
		data.getUnscaledValue("vKfz").setText("nicht ermittelbar");
		data.getUnscaledArray("qPkwÄGeschwKlasse").setLength(0);
		data.getUnscaledArray("qLkwÄGeschwKlasse").setLength(0);

		// Telegrammabhängig die Daten einlesen, im Telegramm nicht vorhandene Daten auf "nicht ermittelbar" lassen
		switch (deBlock.getTyp()) {
			case 49:// Version 0 - 8-Bit-Format
				_qKfz_ = deBlock.getDeData().readUnsignedByteAsInt();
				data.getUnscaledValue("qKfz").set(_qKfz_);
				_qLkwÄ_ = deBlock.getDeData().readUnsignedByteAsInt();
				data.getUnscaledValue("qLkwÄ").set(_qLkwÄ_);
				_vPkwÄ_ = deBlock.getDeData().readUnsignedByteAsInt();
				data.getUnscaledValue("vPkwÄ").set(_vPkwÄ_);
				_vLkwÄ_ = deBlock.getDeData().readUnsignedByteAsInt();
				data.getUnscaledValue("vLkwÄ").set(_vLkwÄ_);

				break;
			case 50:// Version 1 - 8-Bit-Format
				_qKfz_ = deBlock.getDeData().readUnsignedByteAsInt();
				data.getUnscaledValue("qKfz").set(_qKfz_);
				_qLkwÄ_ = deBlock.getDeData().readUnsignedByteAsInt();
				data.getUnscaledValue("qLkwÄ").set(_qLkwÄ_);
				_vPkwÄ_ = deBlock.getDeData().readUnsignedByteAsInt();
				data.getUnscaledValue("vPkwÄ").set(_vPkwÄ_);
				_vLkwÄ_ = deBlock.getDeData().readUnsignedByteAsInt();
				data.getUnscaledValue("vLkwÄ").set(_vLkwÄ_);
				_tNetto_ = deBlock.getDeData().readUnsignedByteAsInt();
				data.getUnscaledValue("tNetto").set(_tNetto_);
				break;
			case 51:// Version 2 - 8-Bit-Format
				_qKfz_ = deBlock.getDeData().readUnsignedByteAsInt();
				data.getUnscaledValue("qKfz").set(_qKfz_);
				_qLkwÄ_ = deBlock.getDeData().readUnsignedByteAsInt();
				data.getUnscaledValue("qLkwÄ").set(_qLkwÄ_);
				_vPkwÄ_ = deBlock.getDeData().readUnsignedByteAsInt();
				data.getUnscaledValue("vPkwÄ").set(_vPkwÄ_);
				_vLkwÄ_ = deBlock.getDeData().readUnsignedByteAsInt();
				data.getUnscaledValue("vLkwÄ").set(_vLkwÄ_);
				_b_ = deBlock.getDeData().readUnsignedByteAsInt();
				if (_b_ != 255) {
					if (_b_ > 100) {
						debug.warning("Belegung von " + _b_ + "% > 100% erhalten! Wert wurde auf 'nicht ermittelbar' geändert!");
					}
					else {
						data.getUnscaledValue("b").set(_b_);
					}
				}
				break;
			case 52:// Version 3 - 8-Bit-Format
				_qKfz_ = deBlock.getDeData().readUnsignedByteAsInt();
				data.getUnscaledValue("qKfz").set(_qKfz_);
				_qLkwÄ_ = deBlock.getDeData().readUnsignedByteAsInt();
				data.getUnscaledValue("qLkwÄ").set(_qLkwÄ_);
				_vPkwÄ_ = deBlock.getDeData().readUnsignedByteAsInt();
				data.getUnscaledValue("vPkwÄ").set(_vPkwÄ_);
				_vLkwÄ_ = deBlock.getDeData().readUnsignedByteAsInt();
				data.getUnscaledValue("vLkwÄ").set(_vLkwÄ_);
				_tNetto_ = deBlock.getDeData().readUnsignedByteAsInt();
				data.getUnscaledValue("tNetto").set(_tNetto_);
				_b_ = deBlock.getDeData().readUnsignedByteAsInt();
				if (_b_ != 255) {
					if (_b_ > 100) {
						debug.warning("Belegung von " + _b_ + "% > 100% erhalten! Wert wurde auf 'nicht ermittelbar' geändert!");
					}
					else {
						data.getUnscaledValue("b").set(_b_);
					}
				}
				_sKfz_ = deBlock.getDeData().readUnsignedByteAsInt();
				data.getUnscaledValue("sKfz").set(_sKfz_);
				_vKfz_ = deBlock.getDeData().readUnsignedByteAsInt();
				data.getUnscaledValue("vKfz").set(_vKfz_);
				break;
			case 53:// Version 4 - 8-Bit-Format
				_qKfz_ = deBlock.getDeData().readUnsignedByteAsInt();
				data.getUnscaledValue("qKfz").set(_qKfz_);
				_qLkwÄ_ = deBlock.getDeData().readUnsignedByteAsInt();
				data.getUnscaledValue("qLkwÄ").set(_qLkwÄ_);
				_vPkwÄ_ = deBlock.getDeData().readUnsignedByteAsInt();
				data.getUnscaledValue("vPkwÄ").set(_vPkwÄ_);
				_vLkwÄ_ = deBlock.getDeData().readUnsignedByteAsInt();
				data.getUnscaledValue("vLkwÄ").set(_vLkwÄ_);
				_tNetto_ = deBlock.getDeData().readUnsignedByteAsInt();
				data.getUnscaledValue("tNetto").set(_tNetto_);
				_b_ = deBlock.getDeData().readUnsignedByteAsInt();
				if (_b_ != 255) {
					if (_b_ > 100) {
						debug.warning("Belegung von " + _b_ + "% > 100% erhalten! Wert wurde auf 'nicht ermittelbar' geändert!");
					}
					else {
						data.getUnscaledValue("b").set(_b_);
					}
				}
				_sKfz_ = deBlock.getDeData().readUnsignedByteAsInt();
				data.getUnscaledValue("sKfz").set(_sKfz_);
				_vKfz_ = deBlock.getDeData().readUnsignedByteAsInt();
				data.getUnscaledValue("vKfz").set(_vKfz_);

				//  Geschwindigkeitsklassen qPkwÄ einlesen
				countQPkwÄGeschwKlasse = deBlock.getDeData().readUnsignedByteAsInt();
				_qPkwÄGeschwKlasse_ = new int[countQPkwÄGeschwKlasse];
				for (int i = 0; i < countQPkwÄGeschwKlasse; i++) {
					_qPkwÄGeschwKlasse_[i] = deBlock.getDeData().readUnsignedByteAsInt();
				}

				_KlassePkw_ = data.getUnscaledArray("qPkwÄGeschwKlasse");
				_KlassePkw_.setLength(countQPkwÄGeschwKlasse);
				for (int i = 0; i < countQPkwÄGeschwKlasse; i++) {
					_KlassePkw_.getValue(i).set(_qPkwÄGeschwKlasse_[i]);
				}

				//  Geschwindigkeitsklassen qPkwÄ einlesen
				countQLkwÄGeschwKlasse = deBlock.getDeData().readUnsignedByteAsInt();
				_qLkwÄGeschwKlasse_ = new int[countQLkwÄGeschwKlasse];
				for (int i = 0; i < countQLkwÄGeschwKlasse; i++) {
					_qLkwÄGeschwKlasse_[i] = deBlock.getDeData().readUnsignedByteAsInt();
				}

				_KlasseLkw_ = data.getUnscaledArray("qLkwÄGeschwKlasse");
				_KlasseLkw_.setLength(countQLkwÄGeschwKlasse);
				for (int i = 0; i < countQLkwÄGeschwKlasse; i++) {
					_KlasseLkw_.getValue(i).set(_qLkwÄGeschwKlasse_[i]);
				}

				break;
			case 113:// Version 0 - 16-Bit-Format
				_qKfz_ = deBlock.getDeData().readUnsignedLowHighByteAsInt();
				data.getUnscaledValue("qKfz").set(_qKfz_);
				_qLkwÄ_ = deBlock.getDeData().readUnsignedLowHighByteAsInt();
				data.getUnscaledValue("qLkwÄ").set(_qLkwÄ_);
				_vPkwÄ_ = deBlock.getDeData().readUnsignedByteAsInt();
				data.getUnscaledValue("vPkwÄ").set(_vPkwÄ_);
				_vLkwÄ_ = deBlock.getDeData().readUnsignedByteAsInt();
				data.getUnscaledValue("vLkwÄ").set(_vLkwÄ_);
				break;
			case 114:// Version 1 - 16-Bit-Format
				_qKfz_ = deBlock.getDeData().readUnsignedLowHighByteAsInt();
				data.getUnscaledValue("qKfz").set(_qKfz_);
				_qLkwÄ_ = deBlock.getDeData().readUnsignedLowHighByteAsInt();
				data.getUnscaledValue("qLkwÄ").set(_qLkwÄ_);
				_vPkwÄ_ = deBlock.getDeData().readUnsignedByteAsInt();
				data.getUnscaledValue("vPkwÄ").set(_vPkwÄ_);
				_vLkwÄ_ = deBlock.getDeData().readUnsignedByteAsInt();
				data.getUnscaledValue("vLkwÄ").set(_vLkwÄ_);
				_tNetto_ = deBlock.getDeData().readUnsignedByteAsInt();
				data.getUnscaledValue("tNetto").set(_tNetto_);
				break;
			case 115:// Version 2 - 16-Bit-Format
				_qKfz_ = deBlock.getDeData().readUnsignedLowHighByteAsInt();
				data.getUnscaledValue("qKfz").set(_qKfz_);
				_qLkwÄ_ = deBlock.getDeData().readUnsignedLowHighByteAsInt();
				data.getUnscaledValue("qLkwÄ").set(_qLkwÄ_);
				_vPkwÄ_ = deBlock.getDeData().readUnsignedByteAsInt();
				data.getUnscaledValue("vPkwÄ").set(_vPkwÄ_);
				_vLkwÄ_ = deBlock.getDeData().readUnsignedByteAsInt();
				data.getUnscaledValue("vLkwÄ").set(_vLkwÄ_);
				_b_ = deBlock.getDeData().readUnsignedByteAsInt();
				if (_b_ != 255) {
					if (_b_ > 100) {
						debug.warning("Belegung von " + _b_ + "% > 100% erhalten! Wert wurde auf 'nicht ermittelbar' geändert!");
					}
					else {
						data.getUnscaledValue("b").set(_b_);
					}
				}
				break;
			case 116:// Version 3 - 16-Bit-Format
				_qKfz_ = deBlock.getDeData().readUnsignedLowHighByteAsInt();
				data.getUnscaledValue("qKfz").set(_qKfz_);
				_qLkwÄ_ = deBlock.getDeData().readUnsignedLowHighByteAsInt();
				data.getUnscaledValue("qLkwÄ").set(_qLkwÄ_);
				_vPkwÄ_ = deBlock.getDeData().readUnsignedByteAsInt();
				data.getUnscaledValue("vPkwÄ").set(_vPkwÄ_);
				_vLkwÄ_ = deBlock.getDeData().readUnsignedByteAsInt();
				data.getUnscaledValue("vLkwÄ").set(_vLkwÄ_);
				_tNetto_ = deBlock.getDeData().readUnsignedByteAsInt();
				data.getUnscaledValue("tNetto").set(_tNetto_);
				_b_ = deBlock.getDeData().readUnsignedByteAsInt();
				if (_b_ != 255) {
					if (_b_ > 100) {
						debug.warning("Belegung von " + _b_ + "% > 100% erhalten! Wert wurde auf 'nicht ermittelbar' geändert!");
					}
					else {
						data.getUnscaledValue("b").set(_b_);
					}
				}
				_sKfz_ = deBlock.getDeData().readUnsignedByteAsInt();
				data.getUnscaledValue("sKfz").set(_sKfz_);
				_vKfz_ = deBlock.getDeData().readUnsignedByteAsInt();
				data.getUnscaledValue("vKfz").set(_vKfz_);
				break;
			case 117:// Version 4 - 16-Bit-Format
				_qKfz_ = deBlock.getDeData().readUnsignedLowHighByteAsInt();
				data.getUnscaledValue("qKfz").set(_qKfz_);
				_qLkwÄ_ = deBlock.getDeData().readUnsignedLowHighByteAsInt();
				data.getUnscaledValue("qLkwÄ").set(_qLkwÄ_);
				_vPkwÄ_ = deBlock.getDeData().readUnsignedByteAsInt();
				data.getUnscaledValue("vPkwÄ").set(_vPkwÄ_);
				_vLkwÄ_ = deBlock.getDeData().readUnsignedByteAsInt();
				data.getUnscaledValue("vLkwÄ").set(_vLkwÄ_);
				_tNetto_ = deBlock.getDeData().readUnsignedByteAsInt();
				data.getUnscaledValue("tNetto").set(_tNetto_);
				_b_ = deBlock.getDeData().readUnsignedByteAsInt();
				if (_b_ != 255) {
					if (_b_ > 100) {
						debug.warning("Belegung von " + _b_ + "% > 100% erhalten! Wert wurde auf 'nicht ermittelbar' geändert!");
					}
					else {
						data.getUnscaledValue("b").set(_b_);
					}
				}
				_sKfz_ = deBlock.getDeData().readUnsignedByteAsInt();
				data.getUnscaledValue("sKfz").set(_sKfz_);
				_vKfz_ = deBlock.getDeData().readUnsignedByteAsInt();
				data.getUnscaledValue("vKfz").set(_vKfz_);

				//  Geschwindigkeitsklassen qPkwÄ einlesen
				countQPkwÄGeschwKlasse = deBlock.getDeData().readUnsignedByteAsInt();
				_qPkwÄGeschwKlasse_ = new int[countQPkwÄGeschwKlasse];
				for (int i = 0; i < countQPkwÄGeschwKlasse; i++) {
					_qPkwÄGeschwKlasse_[i] = deBlock.getDeData().readUnsignedByteAsInt();
				}

				_KlassePkw_ = data.getUnscaledArray("qPkwÄGeschwKlasse");
				_KlassePkw_.setLength(countQPkwÄGeschwKlasse);
				for (int i = 0; i < countQPkwÄGeschwKlasse; i++) {
					_KlassePkw_.getValue(i).set(_qPkwÄGeschwKlasse_[i]);
				}

				//  Geschwindigkeitsklassen qPkwÄ einlesen
				countQLkwÄGeschwKlasse = deBlock.getDeData().readUnsignedByteAsInt();
				_qLkwÄGeschwKlasse_ = new int[countQLkwÄGeschwKlasse];
				for (int i = 0; i < countQLkwÄGeschwKlasse; i++) {
					_qLkwÄGeschwKlasse_[i] = deBlock.getDeData().readUnsignedByteAsInt();
				}

				_KlasseLkw_ = data.getUnscaledArray("qLkwÄGeschwKlasse");
				_KlasseLkw_.setLength(countQLkwÄGeschwKlasse);
				for (int i = 0; i < countQLkwÄGeschwKlasse; i++) {
					_KlasseLkw_.getValue(i).set(_qLkwÄGeschwKlasse_[i]);
				}

				break;
			default:
				debug.warning("Unbekannter LVE-Kurzzeit-Typ (49/113 - 53/117): " + deBlock.getTyp());
		}

		//Ergebnisdatensatz erzeugen...
		resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);
		return resultData;
	}

	public ResultData Fg001Typ049Id004AtgVerkehrsDatenKurzZeitIntervallAspExterneErfassung(DeBlock deBlock, Data data, DataDescription dataDescription) {
		return VerkehrsDatenKurzZeitIntervallAspExterneErfassung(deBlock, data, dataDescription);
	}

	/**
	 * Verarbeitet alls Telegrammversionen der TLS Kurzeit Version 0-4 (Typ 49/113 -53/117).
	 *
	 * @param deBlock         Der zu bearbeitende De-Block.
	 * @param data            Der passende Datensatz für die Ziel Attributgruppe/Aspekt-Kombination.
	 * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
	 *
	 * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
	 */
	private ResultData VerkehrsDatenKurzZeitIntervallAspExterneErfassung(DeBlock deBlock, Data data, DataDescription dataDescription) {
		ResultData resultData = null;

		// Intervalllänge aus IntervalldatenHeader ---------------------------------------------------------------------
		data.getTimeValue("T").setSeconds(_eakConversionModul.getLveIntervalDuration());

		// Art der Mittelwertbildung am DE aus Betriebsparametern ------------------------------------------------------
		switch (_kindOfAverage) {
			case 0:
				data.getUnscaledValue("ArtMittelwertbildung").setText("gleitende Mittelwertbildung");
				break;
			case 1:
				data.getUnscaledValue("ArtMittelwertbildung").setText("arithmetische Mittelwertbildung");
				break;
			default:
				data.getUnscaledValue("ArtMittelwertbildung").setText("unbekannt");
		}

		int _qKfz_;// qKfz-Wert aus Telegramm
		int _qLkw_;// qLkw-Wert aus Telegramm
		int _vPkw_;// vPkwÄ-Wert aus Telegramm
		int _vLkw_;// vLkwÄ-Wert aus Telegramm
		int _tNetto_;// tNetto-Wert aus Telegramm
		int _b_;// Belegungs-Wert aus Telegramm
		int _sKfz_;// Streuung Kfz-Wert aus Telegramm
		int _vgKfz_;// vgKfz-Wert aus Telegramm
		int _qPkw_;// wird berechnet
		int _vKfz_;// wird berechnet

		// Werte jeweils auf Zustand "nicht ermittelbar" setzen.

		String[] valStrings = {"qKfz", "vKfz", "qLkw", "vLkw", "qPkw", "vPkw", "b", "tNetto", "sKfz", "vgKfz"};

		for (int idx = 0; idx < valStrings.length; idx++) {
			data.getItem(valStrings[idx]).getUnscaledValue("Wert").setText("nicht ermittelbar");
			data.getItem(valStrings[idx]).getItem("Status").getItem("Erfassung").getUnscaledValue("NichtErfasst").setText("Nein");
			data.getItem(valStrings[idx]).getItem("Status").getItem("PlFormal").getUnscaledValue("WertMax").setText("Nein");
			data.getItem(valStrings[idx]).getItem("Status").getItem("PlFormal").getUnscaledValue("WertMin").setText("Nein");
			data.getItem(valStrings[idx]).getItem("Status").getItem("PlLogisch").getUnscaledValue("WertMaxLogisch").setText("Nein");
			data.getItem(valStrings[idx]).getItem("Status").getItem("PlLogisch").getUnscaledValue("WertMinLogisch").setText("Nein");
			data.getItem(valStrings[idx]).getItem("Status").getItem("MessWertErsetzung").getUnscaledValue("Implausibel").setText("Nein");
			data.getItem(valStrings[idx]).getItem("Status").getItem("MessWertErsetzung").getUnscaledValue("Interpoliert").setText("Nein");
			data.getItem(valStrings[idx]).getItem("Güte").getUnscaledValue("Index").set(-1);
			data.getItem(valStrings[idx]).getItem("Güte").getUnscaledValue("Verfahren").set(0);
		}

		// Telegrammabhängig die Daten einlesen, im Telegramm nicht vorhandene Daten auf "nicht ermittelbar" lassen
		switch (deBlock.getTyp()) {
			case 49:// Version 0 - 8-Bit-Format
				_qKfz_ = deBlock.getDeData().readUnsignedByteAsInt();
				data.getItem("qKfz").getUnscaledValue("Wert").set(tls8BitToInt(_qKfz_));
				data.getItem("qKfz").getItem("Güte").getScaledValue("Index").set(1.0);
				_qLkw_ = deBlock.getDeData().readUnsignedByteAsInt();
				data.getItem("qLkw").getUnscaledValue("Wert").set(tls8BitToInt(_qLkw_));
				data.getItem("qLkw").getItem("Güte").getScaledValue("Index").set(1.0);
				_vPkw_ = deBlock.getDeData().readUnsignedByteAsInt();
				data.getItem("vPkw").getUnscaledValue("Wert").set(tls8BitToInt(_vPkw_));
				data.getItem("vPkw").getItem("Güte").getScaledValue("Index").set(1.0);
				_vLkw_ = deBlock.getDeData().readUnsignedByteAsInt();
				data.getItem("vLkw").getUnscaledValue("Wert").set(tls8BitToInt(_vLkw_));
				data.getItem("vLkw").getItem("Güte").getScaledValue("Index").set(1.0);

				// Nicht erfasste Werte qPkw und vKfz berechnen
				_qPkw_ = (_qKfz_ - _qLkw_) >= 0 ? (_qKfz_ - _qLkw_) : -1;
				data.getItem("qPkw").getUnscaledValue("Wert").set(tls8BitToInt(_qPkw_));
				data.getItem("qPkw").getItem("Güte").getScaledValue("Index").set(1.0);
				data.getItem("qPkw").getItem("Status").getItem("Erfassung").getUnscaledValue("NichtErfasst").setText("Ja");

				_vKfz_ = (_qLkw_ + _qPkw_) > 0 ? ((_qLkw_ * _vLkw_) + (_qPkw_ * _vPkw_)) / (_qLkw_ + _qPkw_) : -1;
				data.getItem("vKfz").getUnscaledValue("Wert").set(tls8BitToInt(_vKfz_));
				data.getItem("vKfz").getItem("Güte").getScaledValue("Index").set(1.0);
				data.getItem("vKfz").getItem("Status").getItem("Erfassung").getUnscaledValue("NichtErfasst").setText("Ja");

				break;
			case 50:// Version 1 - 8-Bit-Format
				_qKfz_ = deBlock.getDeData().readUnsignedByteAsInt();
				data.getItem("qKfz").getUnscaledValue("Wert").set(tls8BitToInt(_qKfz_));
				data.getItem("qKfz").getItem("Güte").getScaledValue("Index").set(1.0);
				_qLkw_ = deBlock.getDeData().readUnsignedByteAsInt();
				data.getItem("qLkw").getUnscaledValue("Wert").set(tls8BitToInt(_qLkw_));
				data.getItem("qLkw").getItem("Güte").getScaledValue("Index").set(1.0);
				_vPkw_ = deBlock.getDeData().readUnsignedByteAsInt();
				data.getItem("vPkw").getUnscaledValue("Wert").set(tls8BitToInt(_vPkw_));
				data.getItem("vPkw").getItem("Güte").getScaledValue("Index").set(1.0);
				_vLkw_ = deBlock.getDeData().readUnsignedByteAsInt();
				data.getItem("vLkw").getUnscaledValue("Wert").set(tls8BitToInt(_vLkw_));
				data.getItem("vLkw").getItem("Güte").getScaledValue("Index").set(1.0);
				_tNetto_ = deBlock.getDeData().readUnsignedByteAsInt() * 100;// Umrechnung in Millisekunden
				data.getItem("tNetto").getUnscaledValue("Wert").set(_tNetto_);
				data.getItem("tNetto").getItem("Güte").getScaledValue("Index").set(1.0);

				// Nicht erfasste Werte qPkw und vKfz berechnen
				// Nicht erfasste Werte qPkw und vKfz berechnen
				_qPkw_ = (_qKfz_ - _qLkw_) >= 0 ? (_qKfz_ - _qLkw_) : -1;
				data.getItem("qPkw").getUnscaledValue("Wert").set(tls8BitToInt(_qPkw_));
				data.getItem("qPkw").getItem("Güte").getScaledValue("Index").set(1.0);
				data.getItem("qPkw").getItem("Status").getItem("Erfassung").getUnscaledValue("NichtErfasst").setText("Ja");

				_vKfz_ = (_qLkw_ + _qPkw_) > 0 ? ((_qLkw_ * _vLkw_) + (_qPkw_ * _vPkw_)) / (_qLkw_ + _qPkw_) : -1;
				data.getItem("vKfz").getUnscaledValue("Wert").set(tls8BitToInt(_vKfz_));
				data.getItem("vKfz").getItem("Güte").getScaledValue("Index").set(1.0);
				data.getItem("vKfz").getItem("Status").getItem("Erfassung").getUnscaledValue("NichtErfasst").setText("Ja");

				break;
			case 51:// Version 2 - 8-Bit-Format
				_qKfz_ = deBlock.getDeData().readUnsignedByteAsInt();
				data.getItem("qKfz").getUnscaledValue("Wert").set(tls8BitToInt(_qKfz_));
				data.getItem("tNetto").getItem("Güte").getScaledValue("Index").set(1.0);
				_qLkw_ = deBlock.getDeData().readUnsignedByteAsInt();
				data.getItem("qLkw").getUnscaledValue("Wert").set(tls8BitToInt(_qLkw_));
				data.getItem("tNetto").getItem("Güte").getScaledValue("Index").set(1.0);
				_vPkw_ = deBlock.getDeData().readUnsignedByteAsInt();
				data.getItem("vPkw").getUnscaledValue("Wert").set(tls8BitToInt(_vPkw_));
				data.getItem("tNetto").getItem("Güte").getScaledValue("Index").set(1.0);
				_vLkw_ = deBlock.getDeData().readUnsignedByteAsInt();
				data.getItem("vLkw").getUnscaledValue("Wert").set(tls8BitToInt(_vLkw_));
				data.getItem("tNetto").getItem("Güte").getScaledValue("Index").set(1.0);
				_b_ = deBlock.getDeData().readUnsignedByteAsInt();
				if (_b_ != 255) {
					if (_b_ > 100) {
						debug.warning("Belegung von " + _b_ + "% > 100% erhalten! Wert wurde auf 'nicht ermittelbar' geändert!");
					}
					else {
						data.getItem("b").getUnscaledValue("Wert").set(_b_);
						data.getItem("tNetto").getItem("Güte").getScaledValue("Index").set(1.0);
					}
				}

				// Nicht erfasste Werte qPkw und vKfz berechnen
				// Nicht erfasste Werte qPkw und vKfz berechnen
				_qPkw_ = (_qKfz_ - _qLkw_) >= 0 ? (_qKfz_ - _qLkw_) : -1;
				data.getItem("qPkw").getUnscaledValue("Wert").set(tls8BitToInt(_qPkw_));
				data.getItem("qPkw").getItem("Güte").getScaledValue("Index").set(1.0);
				data.getItem("qPkw").getItem("Status").getItem("Erfassung").getUnscaledValue("NichtErfasst").setText("Ja");

				_vKfz_ = (_qLkw_ + _qPkw_) > 0 ? ((_qLkw_ * _vLkw_) + (_qPkw_ * _vPkw_)) / (_qLkw_ + _qPkw_) : -1;
				data.getItem("vKfz").getUnscaledValue("Wert").set(tls8BitToInt(_vKfz_));
				data.getItem("vKfz").getItem("Güte").getScaledValue("Index").set(1.0);
				data.getItem("vKfz").getItem("Status").getItem("Erfassung").getUnscaledValue("NichtErfasst").setText("Ja");

				break;
			case 52:// Version 3 - 8-Bit-Format
				_qKfz_ = deBlock.getDeData().readUnsignedByteAsInt();
				data.getItem("qKfz").getUnscaledValue("Wert").set(tls8BitToInt(_qKfz_));
				data.getItem("qKfz").getItem("Güte").getScaledValue("Index").set(1.0);
				_qLkw_ = deBlock.getDeData().readUnsignedByteAsInt();
				data.getItem("qLkw").getUnscaledValue("Wert").set(tls8BitToInt(_qLkw_));
				data.getItem("qLkw").getItem("Güte").getScaledValue("Index").set(1.0);
				_vPkw_ = deBlock.getDeData().readUnsignedByteAsInt();
				data.getItem("vPkw").getUnscaledValue("Wert").set(tls8BitToInt(_vPkw_));
				data.getItem("vPkw").getItem("Güte").getScaledValue("Index").set(1.0);
				_vLkw_ = deBlock.getDeData().readUnsignedByteAsInt();
				data.getItem("vLkw").getUnscaledValue("Wert").set(tls8BitToInt(_vLkw_));
				data.getItem("vLkw").getItem("Güte").getScaledValue("Index").set(1.0);
				_tNetto_ = deBlock.getDeData().readUnsignedByteAsInt() * 100;// Umrechnung in Millisekunden
				data.getItem("tNetto").getUnscaledValue("Wert").set(_tNetto_);
				data.getItem("tNetto").getItem("Güte").getScaledValue("Index").set(1.0);
				_b_ = deBlock.getDeData().readUnsignedByteAsInt();
				if (_b_ != 255) {
					if (_b_ > 100) {
						debug.warning("Belegung von " + _b_ + "% > 100% erhalten! Wert wurde auf 'nicht ermittelbar' geändert!");
					}
					else {
						data.getItem("b").getUnscaledValue("Wert").set(_b_);
						data.getItem("b").getItem("Güte").getScaledValue("Index").set(1.0);
					}
				}
				_sKfz_ = deBlock.getDeData().readUnsignedByteAsInt();
				data.getItem("sKfz").getUnscaledValue("Wert").set(tls8BitToInt(_sKfz_));
				data.getItem("sKfz").getItem("Güte").getScaledValue("Index").set(1.0);
				_vgKfz_ = deBlock.getDeData().readUnsignedByteAsInt();
				data.getItem("vgKfz").getUnscaledValue("Wert").set(tls8BitToInt(_vgKfz_));
				data.getItem("vgKfz").getItem("Güte").getScaledValue("Index").set(1.0);

				// Nicht erfasste Werte qPkw und vKfz berechnen
				// Nicht erfasste Werte qPkw und vKfz berechnen
				_qPkw_ = (_qKfz_ - _qLkw_) >= 0 ? (_qKfz_ - _qLkw_) : -1;
				data.getItem("qPkw").getUnscaledValue("Wert").set(tls8BitToInt(_qPkw_));
				data.getItem("qPkw").getItem("Güte").getScaledValue("Index").set(1.0);
				data.getItem("qPkw").getItem("Status").getItem("Erfassung").getUnscaledValue("NichtErfasst").setText("Ja");

				_vKfz_ = (_qLkw_ + _qPkw_) > 0 ? ((_qLkw_ * _vLkw_) + (_qPkw_ * _vPkw_)) / (_qLkw_ + _qPkw_) : -1;
				data.getItem("vKfz").getUnscaledValue("Wert").set(tls8BitToInt(_vKfz_));
				data.getItem("vKfz").getItem("Güte").getScaledValue("Index").set(1.0);
				data.getItem("vKfz").getItem("Status").getItem("Erfassung").getUnscaledValue("NichtErfasst").setText("Ja");

				break;

			case 53:// Version 4 - 8-Bit-Format
				_qKfz_ = deBlock.getDeData().readUnsignedByteAsInt();
				data.getItem("qKfz").getUnscaledValue("Wert").set(tls8BitToInt(_qKfz_));
				data.getItem("qKfz").getItem("Güte").getScaledValue("Index").set(1.0);
				_qLkw_ = deBlock.getDeData().readUnsignedByteAsInt();
				data.getItem("qLkw").getUnscaledValue("Wert").set(tls8BitToInt(_qLkw_));
				data.getItem("qLkw").getItem("Güte").getScaledValue("Index").set(1.0);
				_vPkw_ = deBlock.getDeData().readUnsignedByteAsInt();
				data.getItem("vPkw").getUnscaledValue("Wert").set(tls8BitToInt(_vPkw_));
				data.getItem("vPkw").getItem("Güte").getScaledValue("Index").set(1.0);
				_vLkw_ = deBlock.getDeData().readUnsignedByteAsInt();
				data.getItem("vLkw").getUnscaledValue("Wert").set(tls8BitToInt(_vLkw_));
				data.getItem("vLkw").getItem("Güte").getScaledValue("Index").set(1.0);
				_tNetto_ = deBlock.getDeData().readUnsignedByteAsInt() * 100;// Umrechnung in Millisekunden
				data.getItem("tNetto").getUnscaledValue("Wert").set(_tNetto_);
				data.getItem("tNetto").getItem("Güte").getScaledValue("Index").set(1.0);
				_b_ = deBlock.getDeData().readUnsignedByteAsInt();
				if (_b_ != 255) {
					if (_b_ > 100) {
						debug.warning("Belegung von " + _b_ + "% > 100% erhalten! Wert wurde auf 'nicht ermittelbar' geändert!");
					}
					else {
						data.getItem("b").getUnscaledValue("Wert").set(_b_);
						data.getItem("b").getItem("Güte").getScaledValue("Index").set(1.0);
					}
				}
				_sKfz_ = deBlock.getDeData().readUnsignedByteAsInt();
				data.getItem("sKfz").getUnscaledValue("Wert").set(tls8BitToInt(_sKfz_));
				data.getItem("sKfz").getItem("Güte").getScaledValue("Index").set(1.0);
				_vgKfz_ = deBlock.getDeData().readUnsignedByteAsInt();
				data.getItem("vgKfz").getUnscaledValue("Wert").set(tls8BitToInt(_vgKfz_));
				data.getItem("vgKfz").getItem("Güte").getScaledValue("Index").set(1.0);

				// Nicht erfasste Werte qPkw und vKfz berechnen
				// Nicht erfasste Werte qPkw und vKfz berechnen
				_qPkw_ = (_qKfz_ - _qLkw_) >= 0 ? (_qKfz_ - _qLkw_) : -1;
				data.getItem("qPkw").getUnscaledValue("Wert").set(tls8BitToInt(_qPkw_));
				data.getItem("qPkw").getItem("Güte").getScaledValue("Index").set(1.0);
				data.getItem("qPkw").getItem("Status").getItem("Erfassung").getUnscaledValue("NichtErfasst").setText("Ja");

				_vKfz_ = (_qLkw_ + _qPkw_) > 0 ? ((_qLkw_ * _vLkw_) + (_qPkw_ * _vPkw_)) / (_qLkw_ + _qPkw_) : -1;
				data.getItem("vKfz").getUnscaledValue("Wert").set(tls8BitToInt(_vKfz_));
				data.getItem("vKfz").getItem("Güte").getScaledValue("Index").set(1.0);
				data.getItem("vKfz").getItem("Status").getItem("Erfassung").getUnscaledValue("NichtErfasst").setText("Ja");

				break;

			case 113:// Version 0 - 16-Bit-Format
				_qKfz_ = deBlock.getDeData().readUnsignedLowHighByteAsInt();
				data.getItem("qKfz").getUnscaledValue("Wert").set(tls16BitToInt(_qKfz_));
				data.getItem("qKfz").getItem("Güte").getScaledValue("Index").set(1.0);
				_qLkw_ = deBlock.getDeData().readUnsignedLowHighByteAsInt();
				data.getItem("qLkw").getUnscaledValue("Wert").set(tls16BitToInt(_qLkw_));
				data.getItem("qLkw").getItem("Güte").getScaledValue("Index").set(1.0);
				_vPkw_ = deBlock.getDeData().readUnsignedByteAsInt();
				data.getItem("vPkw").getUnscaledValue("Wert").set(tls8BitToInt(_vPkw_));
				data.getItem("vPkw").getItem("Güte").getScaledValue("Index").set(1.0);
				_vLkw_ = deBlock.getDeData().readUnsignedByteAsInt();
				data.getItem("vLkw").getUnscaledValue("Wert").set(tls8BitToInt(_vLkw_));
				data.getItem("vLkw").getItem("Güte").getScaledValue("Index").set(1.0);

				// Nicht erfasste Werte qPkw und vKfz berechnen
				_qPkw_ = (_qKfz_ - _qLkw_) >= 0 ? (_qKfz_ - _qLkw_) : -1;
				data.getItem("qPkw").getUnscaledValue("Wert").set(tls16BitToInt(_qPkw_));
				data.getItem("qPkw").getItem("Güte").getScaledValue("Index").set(1.0);
				data.getItem("qPkw").getItem("Status").getItem("Erfassung").getUnscaledValue("NichtErfasst").setText("Ja");

				_vKfz_ = (_qLkw_ + _qPkw_) > 0 ? ((_qLkw_ * _vLkw_) + (_qPkw_ * _vPkw_)) / (_qLkw_ + _qPkw_) : -1;
				data.getItem("vKfz").getUnscaledValue("Wert").set(tls8BitToInt(_vKfz_));
				data.getItem("vKfz").getItem("Güte").getScaledValue("Index").set(1.0);
				data.getItem("vKfz").getItem("Status").getItem("Erfassung").getUnscaledValue("NichtErfasst").setText("Ja");

				break;
			case 114:// Version 1 - 16-Bit-Format
				_qKfz_ = deBlock.getDeData().readUnsignedLowHighByteAsInt();
				data.getItem("qKfz").getUnscaledValue("Wert").set(tls16BitToInt(_qKfz_));
				data.getItem("qKfz").getItem("Güte").getScaledValue("Index").set(1.0);
				_qLkw_ = deBlock.getDeData().readUnsignedLowHighByteAsInt();
				data.getItem("qLkw").getUnscaledValue("Wert").set(tls16BitToInt(_qLkw_));
				data.getItem("qLkw").getItem("Güte").getScaledValue("Index").set(1.0);
				_vPkw_ = deBlock.getDeData().readUnsignedByteAsInt();
				data.getItem("vPkw").getUnscaledValue("Wert").set(tls8BitToInt(_vPkw_));
				data.getItem("vPkw").getItem("Güte").getScaledValue("Index").set(1.0);
				_vLkw_ = deBlock.getDeData().readUnsignedByteAsInt();
				data.getItem("vLkw").getUnscaledValue("Wert").set(tls8BitToInt(_vLkw_));
				data.getItem("vLkw").getItem("Güte").getScaledValue("Index").set(1.0);
				_tNetto_ = deBlock.getDeData().readUnsignedByteAsInt() * 100;// Umrechnung in Millisekunden
				data.getItem("tNetto").getUnscaledValue("Wert").set(_tNetto_);
				data.getItem("tNetto").getItem("Güte").getScaledValue("Index").set(1.0);

				// Nicht erfasste Werte qPkw und vKfz berechnen
				_qPkw_ = (_qKfz_ - _qLkw_) >= 0 ? (_qKfz_ - _qLkw_) : -1;
				data.getItem("qPkw").getUnscaledValue("Wert").set(tls16BitToInt(_qPkw_));
				data.getItem("qPkw").getItem("Güte").getScaledValue("Index").set(1.0);
				data.getItem("qPkw").getItem("Status").getItem("Erfassung").getUnscaledValue("NichtErfasst").setText("Ja");

				_vKfz_ = (_qLkw_ + _qPkw_) > 0 ? ((_qLkw_ * _vLkw_) + (_qPkw_ * _vPkw_)) / (_qLkw_ + _qPkw_) : -1;
				data.getItem("vKfz").getUnscaledValue("Wert").set(tls8BitToInt(_vKfz_));
				data.getItem("vKfz").getItem("Güte").getScaledValue("Index").set(1.0);
				data.getItem("vKfz").getItem("Status").getItem("Erfassung").getUnscaledValue("NichtErfasst").setText("Ja");

				break;
			case 115:// Version 2 - 16-Bit-Format
				_qKfz_ = deBlock.getDeData().readUnsignedLowHighByteAsInt();
				data.getItem("qKfz").getUnscaledValue("Wert").set(tls16BitToInt(_qKfz_));
				data.getItem("qKfz").getItem("Güte").getScaledValue("Index").set(1.0);
				_qLkw_ = deBlock.getDeData().readUnsignedLowHighByteAsInt();
				data.getItem("qLkw").getUnscaledValue("Wert").set(tls16BitToInt(_qLkw_));
				data.getItem("qLkw").getItem("Güte").getScaledValue("Index").set(1.0);
				_vPkw_ = deBlock.getDeData().readUnsignedByteAsInt();
				data.getItem("vPkw").getUnscaledValue("Wert").set(tls8BitToInt(_vPkw_));
				data.getItem("vPkw").getItem("Güte").getScaledValue("Index").set(1.0);
				_vLkw_ = deBlock.getDeData().readUnsignedByteAsInt();
				data.getItem("vLkw").getUnscaledValue("Wert").set(tls8BitToInt(_vLkw_));
				data.getItem("vLkw").getItem("Güte").getScaledValue("Index").set(1.0);
				_b_ = deBlock.getDeData().readUnsignedByteAsInt();
				if (_b_ != 255) {
					if (_b_ > 100) {
						debug.warning("Belegung von " + _b_ + "% > 100% erhalten! Wert wurde auf 'nicht ermittelbar' geändert!");
					}
					else {
						data.getItem("b").getUnscaledValue("Wert").set(_b_);
						data.getItem("b").getItem("Güte").getScaledValue("Index").set(1.0);
					}
				}

				// Nicht erfasste Werte qPkw und vKfz berechnen
				_qPkw_ = (_qKfz_ - _qLkw_) >= 0 ? (_qKfz_ - _qLkw_) : -1;
				data.getItem("qPkw").getUnscaledValue("Wert").set(tls16BitToInt(_qPkw_));
				data.getItem("qPkw").getItem("Güte").getScaledValue("Index").set(1.0);
				data.getItem("qPkw").getItem("Status").getItem("Erfassung").getUnscaledValue("NichtErfasst").setText("Ja");

				_vKfz_ = (_qLkw_ + _qPkw_) > 0 ? ((_qLkw_ * _vLkw_) + (_qPkw_ * _vPkw_)) / (_qLkw_ + _qPkw_) : -1;
				data.getItem("vKfz").getUnscaledValue("Wert").set(tls8BitToInt(_vKfz_));
				data.getItem("vKfz").getItem("Güte").getScaledValue("Index").set(1.0);
				data.getItem("vKfz").getItem("Status").getItem("Erfassung").getUnscaledValue("NichtErfasst").setText("Ja");
				break;
			case 116:// Version 3 - 16-Bit-Format
				_qKfz_ = deBlock.getDeData().readUnsignedLowHighByteAsInt();
				data.getItem("qKfz").getUnscaledValue("Wert").set(tls16BitToInt(_qKfz_));
				data.getItem("qKfz").getItem("Güte").getScaledValue("Index").set(1.0);
				_qLkw_ = deBlock.getDeData().readUnsignedLowHighByteAsInt();
				data.getItem("qLkw").getUnscaledValue("Wert").set(tls16BitToInt(_qLkw_));
				data.getItem("qLkw").getItem("Güte").getScaledValue("Index").set(1.0);
				_vPkw_ = deBlock.getDeData().readUnsignedByteAsInt();
				data.getItem("vPkw").getUnscaledValue("Wert").set(tls8BitToInt(_vPkw_));
				data.getItem("vPkw").getItem("Güte").getScaledValue("Index").set(1.0);
				_vLkw_ = deBlock.getDeData().readUnsignedByteAsInt();
				data.getItem("vLkw").getUnscaledValue("Wert").set(tls8BitToInt(_vLkw_));
				data.getItem("vLkw").getItem("Güte").getScaledValue("Index").set(1.0);
				_tNetto_ = deBlock.getDeData().readUnsignedByteAsInt() * 100;// Umrechnung in Millisekunden
				data.getItem("tNetto").getUnscaledValue("Wert").set(_tNetto_);
				data.getItem("tNetto").getItem("Güte").getScaledValue("Index").set(1.0);
				_b_ = deBlock.getDeData().readUnsignedByteAsInt();
				if (_b_ != 255) {
					if (_b_ > 100) {
						debug.warning("Belegung von " + _b_ + "% > 100% erhalten! Wert wurde auf 'nicht ermittelbar' geändert!");
					}
					else {
						data.getItem("b").getUnscaledValue("Wert").set(_b_);
						data.getItem("b").getItem("Güte").getScaledValue("Index").set(1.0);
					}
				}
				_sKfz_ = deBlock.getDeData().readUnsignedByteAsInt();
				data.getItem("sKfz").getUnscaledValue("Wert").set(tls8BitToInt(_sKfz_));
				data.getItem("sKfz").getItem("Güte").getScaledValue("Index").set(1.0);
				_vgKfz_ = deBlock.getDeData().readUnsignedByteAsInt();
				data.getItem("vgKfz").getUnscaledValue("Wert").set(tls8BitToInt(_vgKfz_));
				data.getItem("vgKfz").getItem("Güte").getScaledValue("Index").set(1.0);

				// Nicht erfasste Werte qPkw und vKfz berechnen
				_qPkw_ = (_qKfz_ - _qLkw_) >= 0 ? (_qKfz_ - _qLkw_) : -1;
				data.getItem("qPkw").getUnscaledValue("Wert").set(tls16BitToInt(_qPkw_));
				data.getItem("qPkw").getItem("Güte").getScaledValue("Index").set(1.0);
				data.getItem("qPkw").getItem("Status").getItem("Erfassung").getUnscaledValue("NichtErfasst").setText("Ja");

				_vKfz_ = (_qLkw_ + _qPkw_) > 0 ? ((_qLkw_ * _vLkw_) + (_qPkw_ * _vPkw_)) / (_qLkw_ + _qPkw_) : -1;
				data.getItem("vKfz").getUnscaledValue("Wert").set(tls8BitToInt(_vKfz_));
				data.getItem("vKfz").getItem("Güte").getScaledValue("Index").set(1.0);
				data.getItem("vKfz").getItem("Status").getItem("Erfassung").getUnscaledValue("NichtErfasst").setText("Ja");

				break;

			case 117:// Version 4 - 16-Bit-Format
				_qKfz_ = deBlock.getDeData().readUnsignedLowHighByteAsInt();
				data.getItem("qKfz").getUnscaledValue("Wert").set(tls16BitToInt(_qKfz_));
				data.getItem("qKfz").getItem("Güte").getScaledValue("Index").set(1.0);
				_qLkw_ = deBlock.getDeData().readUnsignedLowHighByteAsInt();
				data.getItem("qLkw").getUnscaledValue("Wert").set(tls16BitToInt(_qLkw_));
				data.getItem("qLkw").getItem("Güte").getScaledValue("Index").set(1.0);
				_vPkw_ = deBlock.getDeData().readUnsignedByteAsInt();
				data.getItem("vPkw").getUnscaledValue("Wert").set(tls8BitToInt(_vPkw_));
				data.getItem("vPkw").getItem("Güte").getScaledValue("Index").set(1.0);
				_vLkw_ = deBlock.getDeData().readUnsignedByteAsInt();
				data.getItem("vLkw").getUnscaledValue("Wert").set(tls8BitToInt(_vLkw_));
				data.getItem("vLkw").getItem("Güte").getScaledValue("Index").set(1.0);
				_tNetto_ = deBlock.getDeData().readUnsignedByteAsInt() * 100;// Umrechnung in Millisekunden
				data.getItem("tNetto").getUnscaledValue("Wert").set(_tNetto_);
				data.getItem("tNetto").getItem("Güte").getScaledValue("Index").set(1.0);
				_b_ = deBlock.getDeData().readUnsignedByteAsInt();
				if (_b_ != 255) {
					if (_b_ > 100) {
						debug.warning("Belegung von " + _b_ + "% > 100% erhalten! Wert wurde auf 'nicht ermittelbar' geändert!");
					}
					else {
						data.getItem("b").getUnscaledValue("Wert").set(_b_);
						data.getItem("b").getItem("Güte").getScaledValue("Index").set(1.0);
					}
				}
				_sKfz_ = deBlock.getDeData().readUnsignedByteAsInt();
				data.getItem("sKfz").getUnscaledValue("Wert").set(tls8BitToInt(_sKfz_));
				data.getItem("sKfz").getItem("Güte").getScaledValue("Index").set(1.0);
				_vgKfz_ = deBlock.getDeData().readUnsignedByteAsInt();
				data.getItem("vgKfz").getUnscaledValue("Wert").set(tls8BitToInt(_vgKfz_));
				data.getItem("vgKfz").getItem("Güte").getScaledValue("Index").set(1.0);

				// Nicht erfasste Werte qPkw und vKfz berechnen
				_qPkw_ = (_qKfz_ - _qLkw_) >= 0 ? (_qKfz_ - _qLkw_) : -1;
				data.getItem("qPkw").getUnscaledValue("Wert").set(tls16BitToInt(_qPkw_));
				data.getItem("qPkw").getItem("Güte").getScaledValue("Index").set(1.0);
				data.getItem("qPkw").getItem("Status").getItem("Erfassung").getUnscaledValue("NichtErfasst").setText("Ja");

				_vKfz_ = (_qLkw_ + _qPkw_) > 0 ? ((_qLkw_ * _vLkw_) + (_qPkw_ * _vPkw_)) / (_qLkw_ + _qPkw_) : -1;
				data.getItem("vKfz").getUnscaledValue("Wert").set(tls8BitToInt(_vKfz_));
				data.getItem("vKfz").getItem("Güte").getScaledValue("Index").set(1.0);
				data.getItem("vKfz").getItem("Status").getItem("Erfassung").getUnscaledValue("NichtErfasst").setText("Ja");

				break;

			default:
				debug.warning("Unbekannter LVE-Kurzzeit-Typ (49/113 - 53/117): " + deBlock.getTyp());
		}

		//Ergebnisdatensatz erzeugen...
		resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);
		return resultData;
	}

	/**
	 * Prüft, ob der übergebene Wert == 0xffff ist. In diesem Fall wird -1 zurückgegeben, so dass dieser als unscaledValue
	 * direkt gesetzt werden kann (entspricht dem Zustand "nicht ermittelbar").
	 *
	 * @param value Der zu prüfende Wert (der als 16 Bit vorzeichenlos interpretiert wird)
	 *
	 * @return value, wenn ungleich 0xffff, sonst -1.
	 */
	private int tls16BitToInt(int value) {
		if (value == 0xffff) {
			return -1;
		}
		else {
			return value;
		}
	}

	/**
	 * Prüft, ob der übergebene Wert == 0xff ist. In diesem Fall wird -1 zurückgegeben, so dass dieser als unscaledValue
	 * direkt gesetzt werden kann (entspricht dem Zustand "nicht ermittelbar").
	 *
	 * @param value Der zu prüfende Wert (der als 8 Bit vorzeichenlos interpretiert wird)
	 *
	 * @return value, wenn ungleich 255, sonst -1.
	 */
	private int tls8BitToInt(int value) {
		if (value == 0xff) {
			return -1;
		}
		else {
			return value;
		}
	}

	public ResultData Fg001Typ050Id004AtgTlsLveErgebnisMeldungVersion0Bis4AspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
		return TlsLveErgebnisMeldungVersion0Bis4AspTlsAntwort(deBlock, data, dataDescription);
	}

	public ResultData Fg001Typ050Id004AtgVerkehrsDatenKurzZeitIntervallAspExterneErfassung(DeBlock deBlock, Data data, DataDescription dataDescription) {
		return VerkehrsDatenKurzZeitIntervallAspExterneErfassung(deBlock, data, dataDescription);
	}

	public ResultData Fg001Typ051Id004AtgTlsLveErgebnisMeldungVersion0Bis4AspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
		return TlsLveErgebnisMeldungVersion0Bis4AspTlsAntwort(deBlock, data, dataDescription);
	}

	public ResultData Fg001Typ051Id004AtgVerkehrsDatenKurzZeitIntervallAspExterneErfassung(DeBlock deBlock, Data data, DataDescription dataDescription) {
		return VerkehrsDatenKurzZeitIntervallAspExterneErfassung(deBlock, data, dataDescription);
	}

	public ResultData Fg001Typ052Id004AtgTlsLveErgebnisMeldungVersion0Bis4AspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
		return TlsLveErgebnisMeldungVersion0Bis4AspTlsAntwort(deBlock, data, dataDescription);
	}

	public ResultData Fg001Typ052Id004AtgVerkehrsDatenKurzZeitIntervallAspExterneErfassung(DeBlock deBlock, Data data, DataDescription dataDescription) {
		return VerkehrsDatenKurzZeitIntervallAspExterneErfassung(deBlock, data, dataDescription);
	}

	public ResultData Fg001Typ053Id004AtgTlsLveErgebnisMeldungVersion0Bis4AspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
		return TlsLveErgebnisMeldungVersion0Bis4AspTlsAntwort(deBlock, data, dataDescription);
	}

	public ResultData Fg001Typ053Id004AtgVerkehrsDatenKurzZeitIntervallAspExterneErfassung(DeBlock deBlock, Data data, DataDescription dataDescription) {
		return VerkehrsDatenKurzZeitIntervallAspExterneErfassung(deBlock, data, dataDescription);
	}

	public ResultData Fg001Typ063Id004AtgTlsLveKfzEinzelDatenAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
		ResultData resultData = null;

		// Status-Byte aus Telegramm -----------------------------------------------------------------------------------
		int _Status_ = deBlock.getDeData().readUnsignedByteAsInt();
		data.getUnscaledValue("Status").set(_Status_);

		// FZKlasse-Byte aus Telegramm ---------------------------------------------------------------------------------
		int _FZKlasse_ = deBlock.getDeData().readUnsignedByteAsInt();

		switch (_FZKlasse_) {
			case 1:
			case 2:
			case 3:
			case 4:
			case 5:
			case 6:
			case 7:
			case 8:
			case 9:
			case 10:
			case 11:
			case 32:
			case 33:
			case 64:
				data.getUnscaledValue("FZKlasse").set(_FZKlasse_);
				break;
			default:
				debug.warning("FZKlasse-Wert ungültig : " + _FZKlasse_ + Debug.NEWLINE +
				              "De-Block (Osi7Adresse-DeNummer-FG-Typ-Id): " + deBlock.getLongKey() + " verworfen:" + Debug.NEWLINE +
				              deBlock.toString());
				return resultData;
		}

		// vKfz und Fahrtrichung aus Telegramm -------------------------------------------------------------------------
		int _vKfz_ = deBlock.getDeData().readUnsignedLowHighByteAsInt();
		int _Fahrtrichtung_ = _vKfz_ >>> 15;

		data.getUnscaledValue("vKfz").set(_vKfz_ & 0x7fff);
		data.getUnscaledValue("Fahrtrichtung").set(_Fahrtrichtung_);

		// Belegzeit-Byte aus Telegramm --------------------------------------------------------------------------------
		int _Belegzeit_ = deBlock.getDeData().readUnsignedLowHighByteAsInt();
		data.getUnscaledValue("Belegzeit").set(_Belegzeit_);

		// tNetto-Byte aus Telegramm -----------------------------------------------------------------------------------
		int _tNetto_ = deBlock.getDeData().readUnsignedLowHighByteAsInt();
		data.getUnscaledValue("tNetto").set(_tNetto_);

		// FZLänge-Byte aus Telegramm ----------------------------------------------------------------------------------
		int _FZLänge_ = deBlock.getDeData().readUnsignedByteAsInt();
		data.getUnscaledValue("FZLänge").set(_FZLänge_);

		//Ergebnisdatensatz erzeugen...
		resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);
		return resultData;
	}

	public ResultData Fg001Typ113Id004AtgTlsLveErgebnisMeldungVersion0Bis4AspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
		return TlsLveErgebnisMeldungVersion0Bis4AspTlsAntwort(deBlock, data, dataDescription);
	}

	public ResultData Fg001Typ113Id004AtgVerkehrsDatenKurzZeitIntervallAspExterneErfassung(DeBlock deBlock, Data data, DataDescription dataDescription) {
		return VerkehrsDatenKurzZeitIntervallAspExterneErfassung(deBlock, data, dataDescription);
	}

	public ResultData Fg001Typ114Id004AtgTlsLveErgebnisMeldungVersion0Bis4AspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
		return TlsLveErgebnisMeldungVersion0Bis4AspTlsAntwort(deBlock, data, dataDescription);
	}

	public ResultData Fg001Typ114Id004AtgVerkehrsDatenKurzZeitIntervallAspExterneErfassung(DeBlock deBlock, Data data, DataDescription dataDescription) {
		return VerkehrsDatenKurzZeitIntervallAspExterneErfassung(deBlock, data, dataDescription);
	}

	public ResultData Fg001Typ115Id004AtgTlsLveErgebnisMeldungVersion0Bis4AspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
		return TlsLveErgebnisMeldungVersion0Bis4AspTlsAntwort(deBlock, data, dataDescription);
	}

	public ResultData Fg001Typ115Id004AtgVerkehrsDatenKurzZeitIntervallAspExterneErfassung(DeBlock deBlock, Data data, DataDescription dataDescription) {
		return VerkehrsDatenKurzZeitIntervallAspExterneErfassung(deBlock, data, dataDescription);
	}

	public ResultData Fg001Typ116Id004AtgTlsLveErgebnisMeldungVersion0Bis4AspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
		return TlsLveErgebnisMeldungVersion0Bis4AspTlsAntwort(deBlock, data, dataDescription);
	}

	public ResultData Fg001Typ116Id004AtgVerkehrsDatenKurzZeitIntervallAspExterneErfassung(DeBlock deBlock, Data data, DataDescription dataDescription) {
		return VerkehrsDatenKurzZeitIntervallAspExterneErfassung(deBlock, data, dataDescription);
	}

	public ResultData Fg001Typ117Id004AtgTlsLveErgebnisMeldungVersion0Bis4AspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
		return TlsLveErgebnisMeldungVersion0Bis4AspTlsAntwort(deBlock, data, dataDescription);
	}

	public ResultData Fg001Typ117Id004AtgVerkehrsDatenKurzZeitIntervallAspExterneErfassung(DeBlock deBlock, Data data, DataDescription dataDescription) {
		return VerkehrsDatenKurzZeitIntervallAspExterneErfassung(deBlock, data, dataDescription);
	}
}
